@@ -1,4 +1,16 @@
-"""PGSync Constants."""
+"""
+PGSync Constants.
+
+This module contains constants used in PGSync.
+It includes constants for relationship types, relationship variants,
+node attributes, relationship attributes, relationship foreign keys,
+tg_op, JSONB operators, Elasticsearch/OpenSearch types,
+Elasticsearch/OpenSearch mapping parameters, transform types, default Postgres/MySQL/MariaDB schema,
+built-in schemas, primary key identifier, logical decoding output plugin,
+trigger function, materialized views, primary key delimiter,
+and replication slot patterns.
+"""
+
 import re
 
 # Relationship types
@@ -53,7 +65,7 @@ INSERT = "INSERT"
 DELETE = "DELETE"
 TRUNCATE = "TRUNCATE"
 
-TG_OP = [
+TG_OPS = [
     DELETE,
     INSERT,
     TRUNCATE,
@@ -73,9 +85,11 @@ ELASTICSEARCH_TYPES = [
     "binary",
     "boolean",
     "byte",
+    "completion",
     "constant_keyword",
     "date",
     "date_range",
+    "dense_vector",
     "double",
     "double_range",
     "flattened",
@@ -121,6 +135,7 @@ ELASTICSEARCH_MAPPING_PARAMETERS = [
     "boost",
     "coerce",
     "copy_to",
+    "dimension",
     "doc_values",
     "dynamic",
     "eager_global_ordinals",
@@ -184,6 +199,7 @@ MATERIALIZED_VIEW_COLUMNS = [
     "indices",
     "primary_keys",
     "table_name",
+    "columns",
 ]
 
 # Primary key delimiter
@@ -194,5 +210,5 @@ LOGICAL_SLOT_PREFIX = re.compile(
     r"table\s\"?(?P<schema>[\w-]+)\"?.\"?(?P<table>[\w-]+)\"?:\s(?P<tg_op>[A-Z]+):"  # noqa E501
 )
 LOGICAL_SLOT_SUFFIX = re.compile(
-    '\s(?P<key>"?\w+"?)\[(?P<type>[\w\s]+)\]:(?P<value>[\w\'"\-]+)'
+    r'\s(?P<key>"?\w+"?)\[(?P<type>[\w\s]+)\]:(?P<value>(?:"[^"]*"|\'[^\']*\'|null|\d+e[+-]?\d+|\w+))'
 )
