@@ -1,11 +1,13 @@
 """PGSync Plugin."""
+
 import logging
 import os
+import sys
+import typing as t
 from abc import ABC, abstractmethod
 from importlib import import_module
 from inspect import getmembers, isclass
 from pkgutil import iter_modules
-from typing import Generator, Optional
 
 logger = logging.getLogger(__name__)
 
@@ -14,13 +16,25 @@ class Plugin(ABC):
     """Plugin base class."""
 
     @abstractmethod
-    def transform(self, doc: dict, **kwargs) -> dict:
+    def transform(self, doc: dict, **kwargs: t.Any) -> dict:
         """This must be implemented by all derived classes."""
         pass
 
 
 class Plugins(object):
-    def __init__(self, package: str, names: Optional[list] = None):
+    """
+    A class representing a plugin.
+
+    Args:
+        package (str): The name of the package.
+        names (list, optional): A list of names. Defaults to None.
+
+    Attributes:
+        package (str): The name of the package.
+        names (list): A list of names.
+    """
+
+    def __init__(self, package: str, names: t.Optional[list] = None):
         self.package: str = package
         self.names: list = names or []
         self.reload()
@@ -30,7 +44,14 @@ class Plugins(object):
         self.plugins: list = []
         self._paths: list = []
         logger.debug(f"Reloading plugins from package: {self.package}")
-        self.walk(self.package)
+        # skip in test
+        if "test" not in sys.argv[0]:
+            self.walk(self.package)
+
+        # main plugin ordering
+        self.plugins = sorted(
+            self.plugins, key=lambda x: self.names.index(x.name)
+        )
 
     def walk(self, package: str) -> None:
         """Recursively walk the supplied package and fetch all plugins."""
@@ -58,7 +79,6 @@ class Plugins(object):
             paths.extend([path for path in module.__path__])
 
         for pkg_path in paths:
-
             if pkg_path in self._paths:
                 continue
 
@@ -70,7 +90,7 @@ class Plugins(object):
             ]:
                 self.walk(f"{package}.{pkg}")
 
-    def transform(self, docs: list) -> Generator:
+    def transform(self, docs: t.Iterable[dict]) -> t.Generator:
         """Applies all plugins to each doc."""
         for doc in docs:
             for plugin in self.plugins:
@@ -83,7 +103,7 @@ class Plugins(object):
                     yield
             yield doc
 
-    def auth(self, key: str) -> Optional[str]:
+    def auth(self, key: str) -> t.Optional[str]:
         """Get an auth value from a key."""
         for plugin in self.plugins:
             if hasattr(plugin, "auth"):
