@@ -1,6 +1,8 @@
 """PGSync QueryBuilder."""
+
+import threading
+import typing as t
 from collections import defaultdict
-from typing import Dict, List, Optional
 
 import sqlalchemy as sa
 
@@ -8,9 +10,66 @@ from .base import compiled_query, TupleIdentifierType
 from .constants import OBJECT, ONE_TO_MANY, ONE_TO_ONE, SCALAR
 from .exc import ForeignKeyError
 from .node import Node
+from .settings import IS_MYSQL_COMPAT
 
 
-class QueryBuilder(object):
+def JSON_OBJECT(*args: t.Any) -> sa.sql.functions.Function:
+    """JSON object constructor."""
+    return (
+        sa.func.JSON_OBJECT(*args)
+        if IS_MYSQL_COMPAT
+        else sa.func.JSON_BUILD_OBJECT(*args)
+    )
+
+
+def JSON_ARRAY(*args: t.Any) -> sa.sql.functions.Function:
+    """JSON array constructor."""
+    return (
+        sa.func.JSON_ARRAY(*args)
+        if IS_MYSQL_COMPAT
+        else sa.func.JSON_BUILD_ARRAY(*args)
+    )
+
+
+def JSON_AGG(expr: t.Any) -> sa.sql.functions.Function:
+    """Aggregate into JSON array."""
+    return (
+        sa.func.JSON_ARRAYAGG(sa.distinct(expr))
+        if IS_MYSQL_COMPAT
+        else sa.func.JSON_AGG(expr)
+    )
+
+
+def JSON_TYPE() -> t.Any:
+    """Resulting JSON type to annotate/cast."""
+    return sa.JSON if IS_MYSQL_COMPAT else sa.dialects.postgresql.JSONB
+
+
+def JSON_CAST(expression: t.Any) -> t.Any:
+    """
+    Ensure the expression is treated as JSON by SQLAlchemy.
+    - PG: emit CAST(... AS JSONB)
+    - MySQL/MariaDB: avoid emitting CAST; only annotate type
+    """
+    return (
+        sa.type_coerce(expression, sa.JSON)
+        if IS_MYSQL_COMPAT
+        else sa.cast(expression, sa.dialects.postgresql.JSONB)
+    )
+
+
+def JSON_CONCAT(a: t.Any, b: t.Any) -> t.Any:
+    """
+    Merge/concatenate JSON values (object-merge & array-append).
+    - PG: jsonb || jsonb
+    - MySQL/MariaDB: JSON_MERGE_PRESERVE(a,b) (keeps array elements)
+    """
+    return (
+        sa.func.JSON_MERGE_PRESERVE(a, b) if IS_MYSQL_COMPAT else a.op("||")(b)
+    )
+
+
+class QueryBuilder(threading.local):
     """Query builder."""
 
     def __init__(self, verbose: bool = False):
@@ -19,9 +78,28 @@ class QueryBuilder(object):
         self.isouter: bool = True
         self._cache: dict = {}
 
+    def _eval_expression(
+        self, expression: sa.sql.elements.BinaryExpression
+    ) -> sa.sql.elements.BinaryExpression:
+        if IS_MYSQL_COMPAT:
+            return expression
+        if isinstance(
+            expression.left.type, sa.dialects.postgresql.UUID
+        ) or isinstance(expression.right.type, sa.dialects.postgresql.UUID):
+            if not isinstance(
+                expression.left.type, sa.dialects.postgresql.UUID
+            ) or not isinstance(
+                expression.right.type, sa.dialects.postgresql.UUID
+            ):
+                # handle UUID typed expressions:
+                # psycopg2.errors.UndefinedFunction: operator does not exist: uuid = integer
+                return expression.left is None
+
+        return expression
+
     def _build_filters(
-        self, filters: Dict[str, List[dict]], node: Node
-    ) -> Optional[sa.sql.elements.BooleanClauseList]:
+        self, filters: t.Dict[str, t.List[dict]], node: Node
+    ) -> t.Optional[sa.sql.elements.BooleanClauseList]:
         """
         Build SQLAlchemy filters.
 
@@ -40,17 +118,21 @@ class QueryBuilder(object):
         """
         if filters is not None:
             if filters.get(node.table):
-                clause: list = []
+                clause: t.List = []
                 for values in filters.get(node.table):
-                    where: list = []
+                    where: t.List = []
                     for column, value in values.items():
-                        where.append(node.model.c[column] == value)
+                        where.append(
+                            self._eval_expression(
+                                node.model.c[column] == value
+                            )
+                        )
                     # and clause is applied for composite primary keys
                     clause.append(sa.and_(*where))
                 return sa.or_(*clause)
 
     def _json_build_object(
-        self, columns: list, chunk_size: int = 100
+        self, columns: t.List, chunk_size: int = 100
     ) -> sa.sql.elements.BinaryExpression:
         """
         Tries to get aroud the limitation of JSON_BUILD_OBJECT which
@@ -62,96 +144,144 @@ class QueryBuilder(object):
         at a time.
         """
         i: int = 0
-        expression: sa.sql.elements.BinaryExpression = None
+        expression: t.Optional[sa.sql.elements.BinaryExpression] = None
         while i < len(columns):
-            chunk: list = columns[i : i + chunk_size]
-            if i == 0:
-                expression = sa.cast(
-                    sa.func.JSON_BUILD_OBJECT(*chunk),
-                    sa.dialects.postgresql.JSONB,
-                )
-            else:
-                expression = expression.concat(
-                    sa.cast(
-                        sa.func.JSON_BUILD_OBJECT(*chunk),
-                        sa.dialects.postgresql.JSONB,
-                    )
-                )
+            chunk = columns[i : i + chunk_size]
+            piece = JSON_CAST(JSON_OBJECT(*chunk))
+            expression = (
+                piece if expression is None else JSON_CONCAT(expression, piece)
+            )
             i += chunk_size
 
         if expression is None:
             raise RuntimeError("invalid expression")
-
         return expression
 
     # this is for handling non-through tables
-    def get_foreign_keys(self, node_a: Node, node_b: Node) -> dict:
-        if (node_a, node_b) not in self._cache:
+    def get_foreign_keys(
+        self, node_a: Node, node_b: Node
+    ) -> t.Dict[str, t.List[str]]:
+        """
+        Return all FK columns between node_a and node_b, keyed by fully qualified table name.
+        Example:
+        {
+            "public.child":  ["child_fk1", "child_fk2"],
+            "public.parent": ["id1", "id2"],
+        }
+        """
+        cache_key: t.Tuple[t.Any, t.Any] = (node_a, node_b)
+        if cache_key in self._cache:
+            return self._cache[cache_key]
 
-            foreign_keys: dict = {}
-            # if either offers a foreign_key via relationship, use it!
-            if (
-                node_a.relationship.foreign_key.parent
-                or node_b.relationship.foreign_key.parent
-            ):
-                if node_a.relationship.foreign_key.parent:
-                    foreign_keys[node_a.parent.name] = sorted(
-                        node_a.relationship.foreign_key.parent
-                    )
-                    foreign_keys[node_a.name] = sorted(
-                        node_a.relationship.foreign_key.child
-                    )
-                if node_b.relationship.foreign_key.parent:
-                    foreign_keys[node_b.parent.name] = sorted(
-                        node_b.relationship.foreign_key.parent
-                    )
-                    foreign_keys[node_b.name] = sorted(
-                        node_b.relationship.foreign_key.child
-                    )
+        fkeys: t.MutableMapping[str, t.List[str]] = defaultdict(list)
 
-            else:
-                fkeys: dict = defaultdict(list)
-                if node_a.model.foreign_keys:
-                    for key in node_a.model.original.foreign_keys:
-                        if key._table_key() == str(node_b.model.original):
-                            fkeys[
-                                f"{key.parent.table.schema}."
-                                f"{key.parent.table.name}"
-                            ].append(str(key.parent.name))
-                            fkeys[
-                                f"{key.column.table.schema}."
-                                f"{key.column.table.name}"
-                            ].append(str(key.column.name))
-                if not fkeys:
-                    if node_b.model.original.foreign_keys:
-                        for key in node_b.model.original.foreign_keys:
-                            if key._table_key() == str(node_a.model.original):
-                                fkeys[
-                                    f"{key.parent.table.schema}."
-                                    f"{key.parent.table.name}"
-                                ].append(str(key.parent.name))
-                                fkeys[
-                                    f"{key.column.table.schema}."
-                                    f"{key.column.table.name}"
-                                ].append(str(key.column.name))
-                if not fkeys:
-                    raise ForeignKeyError(
-                        f"No foreign key relationship between "
-                        f"{node_a.model.original} and {node_b.model.original}"
-                    )
+        def add(table_key: t.Optional[str], col: t.Optional[str]) -> None:
+            if not table_key or not col:
+                return
+            if col not in fkeys[table_key]:
+                fkeys[table_key].append(col)
 
-                for table, columns in fkeys.items():
-                    foreign_keys[table] = columns
+        def qname(sa_table: t.Any) -> t.Optional[str]:
+            """schema-qualified table name from a SQLAlchemy Table (or None)."""
+            if sa_table is None:
+                return None
+            schema = getattr(sa_table, "schema", None)
+            name = getattr(sa_table, "name", None) or getattr(
+                sa_table, "key", None
+            )
+            if not name:
+                return None
+            return f"{schema}.{name}" if schema else str(name)
 
-            self._cache[(node_a, node_b)] = foreign_keys
+        def node_table_key(
+            node: t.Any, *, prefer_parent: bool
+        ) -> t.Optional[str]:
+            """Best-effort table key from node or its parent; never raises."""
+            if prefer_parent:
+                parent = getattr(node, "parent", None)
+                parent_name = getattr(parent, "name", None)
+                if parent_name:
+                    return str(parent_name)
+            node_name = getattr(node, "name", None)
+            if node_name:
+                return str(node_name)
+            # fallback to sqlalchemy table
+            return qname(
+                getattr(getattr(node, "model", None), "original", None)
+            )
 
-        return self._cache[(node_a, node_b)]
+        # merge relationship provided hints (if any); do NOT short-circuit
+        for node in (node_a, node_b):
+            rel_fk = getattr(
+                getattr(node, "relationship", None), "foreign_key", None
+            )
+            if not rel_fk:
+                continue
+
+            parent_tbl_key = node_table_key(node, prefer_parent=True)
+            child_tbl_key = node_table_key(node, prefer_parent=False)
+
+            def merge_side(
+                side_obj: t.Any, fallback_tbl_key: t.Optional[str]
+            ) -> None:
+                # accept dict[str, list[str]] (preferred), or an iterable[str], or a single str
+                if isinstance(side_obj, dict):
+                    for tbl, cols in side_obj.items():
+                        tkey = str(tbl)
+                        for c in cols or []:
+                            add(tkey, str(c))
+                elif isinstance(side_obj, (list, tuple, set)):
+                    for c in side_obj:
+                        add(fallback_tbl_key, str(c))
+                elif isinstance(side_obj, str):
+                    add(fallback_tbl_key, side_obj)
+
+            # parent table cols
+            merge_side(getattr(rel_fk, "parent", None), parent_tbl_key)
+            # child table cols
+            merge_side(getattr(rel_fk, "child", None), child_tbl_key)
+
+        # SQLAlchemy introspection in both directions (A -> B and B -> A)
+        A = getattr(getattr(node_a, "model", None), "original", None)
+        B = getattr(getattr(node_b, "model", None), "original", None)
+
+        A_q = qname(A)
+        B_q = qname(B)
+
+        # helper to compare tables
+        def same_table(t1: t.Any, t2: t.Any) -> bool:
+            return qname(t1) is not None and qname(t1) == qname(t2)
+
+        if A is not None and B is not None:
+            for fk in getattr(A, "foreign_keys", []):
+                # does A have an FK pointing to B?
+                if same_table(getattr(fk, "column", None).table, B):
+                    # child col in A
+                    add(qname(fk.parent.table), str(fk.parent.name))
+                    # parent col in B
+                    add(qname(fk.column.table), str(fk.column.name))
+
+            for fk in getattr(B, "foreign_keys", []):
+                # does B have an FK pointing to A?
+                if same_table(getattr(fk, "column", None).table, A):
+                    # child col in B
+                    add(qname(fk.parent.table), str(fk.parent.name))
+                    # parent col in A
+                    add(qname(fk.column.table), str(fk.column.name))
+
+        if not fkeys:
+            raise ForeignKeyError(
+                f"No foreign key relationship between {A_q or node_a} and {B_q or node_b}"
+            )
+
+        result: t.Dict[str, t.List[str]] = dict(fkeys)
+        self._cache[cache_key] = result
+        return result
 
     def _get_foreign_keys(self, node_a: Node, node_b: Node) -> dict:
         """This is for handling through nodes."""
         if (node_a, node_b) not in self._cache:
             if node_a.relationship.throughs or node_b.relationship.throughs:
-
                 if node_a.relationship.throughs:
                     through: Node = node_a.relationship.throughs[0]
 
@@ -178,7 +308,7 @@ class QueryBuilder(object):
 
     def _get_column_foreign_keys(
         self,
-        columns: List[str],
+        columns: t.List[str],
         foreign_keys: dict,
         table: str = None,
         schema: str = None,
@@ -197,9 +327,9 @@ class QueryBuilder(object):
         """
         # TODO: normalize this elsewhere
         try:
-            column_names: List[str] = [column.name for column in columns]
+            column_names: t.List[str] = [column.name for column in columns]
         except AttributeError:
-            column_names: List[str] = [column for column in columns]
+            column_names: t.List[str] = [column for column in columns]
 
         if table is None:
             for table, cols in foreign_keys.items():
@@ -217,42 +347,36 @@ class QueryBuilder(object):
     def _get_child_keys(
         self, node: Node, params: dict
     ) -> sa.sql.elements.Label:
-        row = sa.cast(
-            sa.func.JSON_BUILD_OBJECT(
+        row = JSON_CAST(
+            JSON_OBJECT(
                 node.table,
                 params,
-            ),
-            sa.dialects.postgresql.JSONB,
+            )
         )
         for child in node.children:
             if (
                 not child.parent.relationship.throughs
                 and child.parent.relationship.type == ONE_TO_MANY
             ):
-                row = row.concat(
-                    sa.cast(
-                        sa.func.JSON_AGG(child._subquery.c._keys),
-                        sa.dialects.postgresql.JSONB,
-                    )
+                row = JSON_CONCAT(
+                    row, JSON_CAST(JSON_AGG(child._subquery.c._keys))
                 )
             else:
-                row = row.concat(
-                    sa.cast(
-                        sa.func.JSON_BUILD_ARRAY(child._subquery.c._keys),
-                        sa.dialects.postgresql.JSONB,
-                    )
+                row = JSON_CONCAT(
+                    row, JSON_CAST(JSON_ARRAY(child._subquery.c._keys))
                 )
+
         return row.label("_keys")
 
     def _root(
         self,
         node: Node,
-        txmin: Optional[int] = None,
-        txmax: Optional[int] = None,
-        ctid: Optional[dict] = None,
+        txmin: t.Optional[int] = None,
+        txmax: t.Optional[int] = None,
+        ctid: t.Optional[dict] = None,
     ) -> None:
         columns = [
-            sa.func.JSON_BUILD_ARRAY(
+            JSON_ARRAY(
                 *[
                     child._subquery.c._keys
                     for child in node.children
@@ -265,7 +389,7 @@ class QueryBuilder(object):
             self._json_build_object(node.columns),
             *node.primary_keys,
         ]
-        node._subquery = sa.select(columns)
+        node._subquery = sa.select(*columns)
 
         if self.from_obj is not None:
             node._subquery = node._subquery.select_from(self.from_obj)
@@ -275,8 +399,8 @@ class QueryBuilder(object):
             for page, rows in ctid.items():
                 subquery.append(
                     sa.select(
-                        [
-                            sa.cast(
+                        *[
+                            JSON_CAST(
                                 sa.literal_column(f"'({page},'")
                                 .concat(sa.column("s"))
                                 .concat(")"),
@@ -325,21 +449,20 @@ class QueryBuilder(object):
                 < txmax
             )
 
+        # NB: only apply filters to the root node
         if node._filters:
             node._subquery = node._subquery.where(sa.and_(*node._filters))
         node._subquery = node._subquery.alias()
 
         if not node.is_root:
-            node._subquery = node._subquery.lateral()
+            if not IS_MYSQL_COMPAT:
+                node._subquery = node._subquery.lateral()
 
     def _children(self, node: Node) -> None:
-
         for child in node.children:
-
-            onclause: List = []
+            onclause: t.List = []
 
             if child.relationship.throughs:
-
                 child.parent.columns.extend(
                     [
                         child.label,
@@ -370,7 +493,6 @@ class QueryBuilder(object):
                     )
 
             else:
-
                 child.parent.columns.extend(
                     [
                         child.label,
@@ -402,10 +524,9 @@ class QueryBuilder(object):
                 self.from_obj = child.parent.model
 
             if child._filters:
-                self.isouter = False
+
                 for _filter in child._filters:
                     if isinstance(_filter, sa.sql.elements.BinaryExpression):
-
                         for column in _filter._orig:
                             if hasattr(column, "value"):
                                 _column = child._subquery.c
@@ -421,7 +542,6 @@ class QueryBuilder(object):
                         sa.sql.elements.BooleanClauseList,
                     ):
                         for clause in _filter.clauses:
-
                             for column in clause._orig:
                                 if hasattr(column, "value"):
                                     _column = child._subquery.c
@@ -445,17 +565,19 @@ class QueryBuilder(object):
             )
 
     def _through(self, node: Node) -> None:  # noqa: C901
-
         through: Node = node.relationship.throughs[0]
-        foreign_keys: dict = self.get_foreign_keys(node, through)
+        # base: fks from through -> node
+        base = self.get_foreign_keys(through, node)
+        foreign_keys: t.Dict[str, t.List[str]] = {
+            k: list(v) for k, v in base.items()
+        }
 
-        for key, values in self.get_foreign_keys(through, node.parent).items():
-            if key in foreign_keys:
-                for value in values:
-                    if value not in foreign_keys[key]:
-                        foreign_keys[key].append(value)
+        for table, cols in self.get_foreign_keys(through, node.parent).items():
+            if table not in foreign_keys:
                 continue
-            foreign_keys[key] = values
+            dst = foreign_keys[table]
+            # extend uniquely and preserve order
+            dst.extend([col for col in cols if col not in dst])
 
         foreign_key_columns: list = self._get_column_foreign_keys(
             node.model.columns,
@@ -467,14 +589,14 @@ class QueryBuilder(object):
         params: list = []
         for foreign_key_column in foreign_key_columns:
             params.append(
-                sa.func.JSON_BUILD_OBJECT(
+                JSON_OBJECT(
                     str(foreign_key_column),
-                    sa.func.JSON_BUILD_ARRAY(node.model.c[foreign_key_column]),
+                    JSON_ARRAY(node.model.c[foreign_key_column]),
                 )
             )
 
         _keys: sa.sql.elements.Label = self._get_child_keys(
-            node, sa.func.JSON_BUILD_ARRAY(*params).label("_keys")
+            node, JSON_ARRAY(*params).label("_keys")
         )
 
         columns = [_keys]
@@ -482,12 +604,10 @@ class QueryBuilder(object):
         if node.relationship.variant == SCALAR:
             columns.append(node.columns[1].label("anon"))
         elif node.relationship.variant == OBJECT:
-
             if node.relationship.type == ONE_TO_ONE:
-
                 if not node.children:
                     columns.append(
-                        sa.func.JSON_BUILD_OBJECT(
+                        JSON_OBJECT(
                             node.columns[0],
                             node.columns[1],
                         ).label("anon")
@@ -511,11 +631,9 @@ class QueryBuilder(object):
         from_obj = None
 
         for child in node.children:
-
             onclause = []
 
             if child.relationship.throughs:
-
                 child_through: Node = child.relationship.throughs[0]
                 child_foreign_keys: dict = self.get_foreign_keys(
                     child, child_through
@@ -539,7 +657,6 @@ class QueryBuilder(object):
                 )
 
             else:
-
                 child_foreign_keys: dict = self.get_foreign_keys(
                     child.parent, child
                 )
@@ -560,11 +677,9 @@ class QueryBuilder(object):
                 from_obj = node.model
 
             if child._filters:
-                self.isouter = False
 
                 for _filter in child._filters:
                     if isinstance(_filter, sa.sql.elements.BinaryExpression):
-
                         for column in _filter._orig:
                             if hasattr(column, "value"):
                                 _column = child._subquery.c
@@ -583,7 +698,6 @@ class QueryBuilder(object):
                         sa.sql.elements.BooleanClauseList,
                     ):
                         for clause in _filter.clauses:
-
                             for column in clause._orig:
                                 if hasattr(column, "value"):
                                     _column = child._subquery.c
@@ -601,11 +715,11 @@ class QueryBuilder(object):
                 isouter=self.isouter,
             )
 
-        outer_subquery = sa.select(columns)
+        outer_subquery = sa.select(*columns)
 
         parent_foreign_key_columns: list = self._get_column_foreign_keys(
             through.columns,
-            foreign_keys,
+            base,
             schema=node.schema,
         )
         where: list = []
@@ -622,7 +736,10 @@ class QueryBuilder(object):
         if from_obj is not None:
             outer_subquery = outer_subquery.select_from(from_obj)
 
-        outer_subquery = outer_subquery.alias().lateral()
+        if IS_MYSQL_COMPAT:
+            outer_subquery = outer_subquery.alias()
+        else:
+            outer_subquery = outer_subquery.alias().lateral()
 
         if self.verbose:
             compiled_query(outer_subquery, "Outer subquery")
@@ -630,39 +747,35 @@ class QueryBuilder(object):
         params = []
         for primary_key in through.model.primary_keys:
             params.append(
-                sa.func.JSON_BUILD_OBJECT(
+                JSON_OBJECT(
                     str(primary_key),
-                    sa.func.JSON_BUILD_ARRAY(through.model.c[primary_key]),
+                    JSON_ARRAY(through.model.c[primary_key]),
                 )
             )
 
-        through_keys = sa.cast(
-            sa.func.JSON_BUILD_OBJECT(
+        through_keys = JSON_CAST(
+            JSON_OBJECT(
                 node.relationship.throughs[0].table,
-                sa.func.JSON_BUILD_ARRAY(*params),
+                JSON_ARRAY(*params),
             ),
-            sa.dialects.postgresql.JSONB,
         )
 
         # book author through table
-        _keys = sa.func.JSON_AGG(
-            sa.cast(
-                outer_subquery.c._keys,
-                sa.dialects.postgresql.JSONB,
-            ).concat(through_keys)
+        _keys = JSON_AGG(
+            JSON_CONCAT(JSON_CAST(outer_subquery.c._keys), through_keys)
         ).label("_keys")
 
         left_foreign_keys = foreign_keys[node.name]
         right_foreign_keys: list = self._get_column_foreign_keys(
             through.columns,
-            foreign_keys,
+            base,
             table=through.table,
             schema=node.schema,
         )
 
         columns = [
             _keys,
-            sa.func.JSON_AGG(outer_subquery.c.anon).label(node.label),
+            JSON_AGG(outer_subquery.c.anon).label(node.label),
         ]
 
         foreign_keys: dict = self.get_foreign_keys(node.parent, through)
@@ -670,7 +783,7 @@ class QueryBuilder(object):
         for column in foreign_keys[through.name]:
             columns.append(through.model.c[str(column)])
 
-        inner_subquery = sa.select(columns)
+        inner_subquery = sa.select(*columns)
 
         if self.verbose:
             compiled_query(inner_subquery, "Inner subquery")
@@ -682,9 +795,6 @@ class QueryBuilder(object):
                 == through.model.c[right_foreign_keys[i]]
             )
 
-        if node._filters:
-            self.isouter = False
-
         op = sa.and_
         if node.table == node.parent.table:
             op = sa.or_
@@ -696,8 +806,10 @@ class QueryBuilder(object):
         )
 
         node._subquery = inner_subquery.select_from(from_obj)
-        if node._filters:
-            node._subquery = node._subquery.where(sa.and_(*node._filters))
+
+        # NB do not apply filters to the child node as they are applied to the parent
+        # if node._filters:
+        #     node._subquery = node._subquery.where(sa.and_(*node._filters))
 
         node._subquery = node._subquery.group_by(
             *[through.model.c[column] for column in foreign_keys[through.name]]
@@ -708,18 +820,17 @@ class QueryBuilder(object):
 
         node._subquery = node._subquery.alias()
         if not node.is_root:
-            node._subquery = node._subquery.lateral()
+            if not IS_MYSQL_COMPAT:
+                node._subquery = node._subquery.lateral()
 
     def _non_through(self, node: Node) -> None:  # noqa: C901
-
         from_obj = None
 
         for child in node.children:
-
-            onclause: list = []
+            onclause: t.List = []
 
             foreign_keys: dict = self._get_foreign_keys(node, child)
-            table: Optional[str] = (
+            table: t.Optional[str] = (
                 child.relationship.throughs[0].table
                 if child.relationship.throughs
                 else None
@@ -741,7 +852,6 @@ class QueryBuilder(object):
                 from_obj = node.model
 
             if child._filters:
-                self.isouter = False
 
                 for _filter in child._filters:
                     if isinstance(_filter, sa.sql.elements.BinaryExpression):
@@ -760,7 +870,6 @@ class QueryBuilder(object):
                         sa.sql.elements.BooleanClauseList,
                     ):
                         for clause in _filter.clauses:
-
                             for column in clause._orig:
                                 if hasattr(column, "value"):
                                     _column = child._subquery.c
@@ -772,11 +881,10 @@ class QueryBuilder(object):
                                             == column.value
                                         )
 
-            isouter = not (len(child._filters) > 0)
             from_obj = from_obj.join(
                 child._subquery,
                 onclause=sa.and_(*onclause),
-                isouter=isouter,
+                isouter=self.isouter,
             )
 
         foreign_keys: dict = self.get_foreign_keys(node.parent, node)
@@ -794,9 +902,7 @@ class QueryBuilder(object):
                 params.extend(
                     [
                         str(primary_key.name),
-                        sa.func.JSON_BUILD_ARRAY(
-                            node.model.c[primary_key.name]
-                        ),
+                        JSON_ARRAY(node.model.c[primary_key.name]),
                     ]
                 )
         else:
@@ -812,10 +918,10 @@ class QueryBuilder(object):
             _keys = self._get_child_keys(node, self._json_build_object(params))
         elif node.relationship.type == ONE_TO_MANY:
             _keys = self._get_child_keys(
-                node, sa.func.JSON_AGG(self._json_build_object(params))
+                node, JSON_AGG(self._json_build_object(params))
             )
 
-        columns: List = [_keys]
+        columns: t.List = [_keys]
 
         if node.relationship.variant == SCALAR:
             # TODO: Raise exception here if the number of columns > 1
@@ -823,9 +929,7 @@ class QueryBuilder(object):
                 columns.append(node.model.c[node.columns[0]].label(node.label))
             elif node.relationship.type == ONE_TO_MANY:
                 columns.append(
-                    sa.func.JSON_AGG(node.model.c[node.columns[0]]).label(
-                        node.label
-                    )
+                    JSON_AGG(node.model.c[node.columns[0]]).label(node.label)
                 )
         elif node.relationship.variant == OBJECT:
             if node.relationship.type == ONE_TO_ONE:
@@ -834,15 +938,15 @@ class QueryBuilder(object):
                 )
             elif node.relationship.type == ONE_TO_MANY:
                 columns.append(
-                    sa.func.JSON_AGG(
-                        self._json_build_object(node.columns)
-                    ).label(node.label)
+                    JSON_AGG(self._json_build_object(node.columns)).label(
+                        node.label
+                    )
                 )
 
         for column in foreign_key_columns:
             columns.append(node.model.c[column])
 
-        node._subquery = sa.select(columns)
+        node._subquery = sa.select(*columns)
 
         if from_obj is not None:
             node._subquery = node._subquery.select_from(from_obj)
@@ -861,8 +965,9 @@ class QueryBuilder(object):
             )
         node._subquery = node._subquery.where(sa.and_(*where))
 
-        if node._filters:
-            node._subquery = node._subquery.where(sa.and_(*node._filters))
+        # NB do not apply filters to the child node as they are applied to the parent
+        # if node._filters:
+        #     node._subquery = node._subquery.where(sa.and_(*node._filters))
 
         if node.relationship.type == ONE_TO_MANY:
             node._subquery = node._subquery.group_by(
@@ -871,15 +976,16 @@ class QueryBuilder(object):
         node._subquery = node._subquery.alias()
 
         if not node.is_root:
-            node._subquery = node._subquery.lateral()
+            if not IS_MYSQL_COMPAT:
+                node._subquery = node._subquery.lateral()
 
     def build_queries(
         self,
         node: Node,
-        filters: Optional[dict] = None,
-        txmin: Optional[int] = None,
-        txmax: Optional[int] = None,
-        ctid: Optional[dict] = None,
+        filters: t.Optional[dict] = None,
+        txmin: t.Optional[int] = None,
+        txmax: t.Optional[int] = None,
+        ctid: t.Optional[dict] = None,
     ) -> None:
         """Build node query."""
         self.from_obj = None
