@@ -1,6 +1,7 @@
 """PGSync Transform."""
+
 import logging
-from typing import Any, Dict, Optional
+import typing as t
 
 from .constants import (  # noqa
     CONCAT_TRANSFORM,
@@ -26,13 +27,24 @@ class Transform(object):
 
     @classmethod
     def _rename(
-        cls, data: dict, nodes: dict, result: Optional[dict] = None
+        cls, data: dict, nodes: dict, result: t.Optional[dict] = None
     ) -> dict:
-        """Rename keys in a nested dictionary based on transform_node.
-        "rename": {
-            "id": "publisher_id",
-            "name": "publisher_name"
-        },
+        """
+        Rename keys in a nested dictionary based on transform_node.
+
+        example:
+            "rename": {
+                "id": "publisher_id",
+                "name": "publisher_name"
+            }
+
+        Args:
+            data (dict): The dictionary to rename keys in.
+            nodes (dict): A dictionary containing the keys to rename and their new names.
+            result (dict, optional): The resulting dictionary after renaming the keys. Defaults to None.
+
+        Returns:
+            dict: The resulting dictionary after renaming the keys.
         """
         result = result or {}
         if isinstance(data, dict):
@@ -64,7 +76,7 @@ class Transform(object):
 
     @classmethod
     def _concat(
-        cls, data: dict, nodes: dict, result: Optional[dict] = None
+        cls, data: dict, nodes: dict, result: t.Optional[dict] = None
     ) -> dict:
         """Concatenate column values into a new field
         {
@@ -203,7 +215,7 @@ class Transform(object):
                 if isinstance(value, dict):
                     target3.append({key: value})
                 elif isinstance(value, list):
-                    _value: Dict[Any, Any] = {}
+                    _value: t.Dict[t.Any, t.Any] = {}
                     for v in value:
                         for _k, _v in v.items():
                             _value.setdefault(_k, [])
