@@ -1,39 +1,49 @@
 """Generic fixtures for PGSync tests."""
+
 import logging
 import os
 
 import pytest
 import sqlalchemy as sa
-from sqlalchemy.ext.declarative import declarative_base
-from sqlalchemy.orm import sessionmaker
+from sqlalchemy.orm import DeclarativeBase, Mapped, mapped_column, sessionmaker
+from sqlalchemy.pool import NullPool
 from sqlalchemy.schema import UniqueConstraint
 
 from pgsync.base import Base, create_database, drop_database
 from pgsync.constants import DEFAULT_SCHEMA
+from pgsync.settings import IS_MYSQL_COMPAT
 from pgsync.singleton import Singleton
 from pgsync.sync import Sync
-from pgsync.urls import get_postgres_url
+from pgsync.urls import get_database_url
 
 logging.getLogger("faker").setLevel(logging.ERROR)
 
 
 @pytest.fixture(scope="session")
 def base():
-    return declarative_base()
+    class Base(DeclarativeBase):
+        pass
+
+    return Base
 
 
 @pytest.fixture(scope="session")
 def dns():
-    return get_postgres_url("testdb")
+    return get_database_url("testdb")
 
 
 @pytest.fixture(scope="session")
 def engine(dns):
-    engine = sa.create_engine(dns)
+    # Use NullPool for tests to avoid connection exhaustion
+
+    engine = sa.create_engine(
+        dns,
+        poolclass=NullPool,
+    )
     drop_database("testdb")
     create_database("testdb")
     yield engine
-    engine.dispose()
+    engine.dispose(close=True)
 
 
 @pytest.fixture(scope="session")
@@ -41,6 +51,8 @@ def connection(engine):
     conn = engine.connect()
     yield conn
     conn.close()
+    # Force cleanup of any remaining connections
+    engine.dispose(close=True)
 
 
 @pytest.fixture(scope="session")
@@ -48,7 +60,8 @@ def session(connection):
     Session = sessionmaker(bind=connection, autoflush=True)
     session = Session()
     yield session
-    session.close_all()
+    session.close()
+    Session.close_all()
 
 
 @pytest.fixture(scope="function")
@@ -61,13 +74,29 @@ def sync():
         }
     )
     yield _sync
-    _sync.logical_slot_get_changes(
-        f"{_sync.database}_testdb",
-        upto_nchanges=None,
-    )
-    _sync.engine.connect().close()
-    _sync.engine.dispose()
-    _sync.session.close()
+    # Cleanup
+    try:
+        if not IS_MYSQL_COMPAT:
+            _sync.logical_slot_get_changes(
+                f"{_sync.database}_testdb",
+                upto_nchanges=None,
+            )
+    except Exception:
+        pass
+
+    # Close all connections properly
+    try:
+        _sync.close_session()
+    except Exception:
+        pass
+
+    try:
+        # Don't create new connection, just dispose the engine
+        _sync.engine.dispose(close=True)
+    except Exception:
+        pass
+
+    # Clear singleton instances
     Singleton._instances = {}
 
 
@@ -97,10 +126,12 @@ def city_cls(base, country_cls):
     class City(base):
         __tablename__ = "city"
         __table_args__ = (UniqueConstraint("name", "country_id"),)
-        id = sa.Column(sa.Integer, primary_key=True)
-        name = sa.Column(sa.String, nullable=False)
-        country_id = sa.Column(sa.Integer, sa.ForeignKey(country_cls.id))
-        country = sa.orm.relationship(
+        id: Mapped[int] = mapped_column(sa.Integer, primary_key=True)
+        name: Mapped[str] = mapped_column(sa.String(256), nullable=False)
+        country_id: Mapped[int] = mapped_column(
+            sa.Integer, sa.ForeignKey(country_cls.id)
+        )
+        country: Mapped[country_cls] = sa.orm.relationship(
             country_cls, backref=sa.orm.backref("countries")
         )
 
@@ -112,10 +143,12 @@ def country_cls(base, continent_cls):
     class Country(base):
         __tablename__ = "country"
         __table_args__ = (UniqueConstraint("name", "continent_id"),)
-        id = sa.Column(sa.Integer, primary_key=True)
-        name = sa.Column(sa.String, nullable=False)
-        continent_id = sa.Column(sa.Integer, sa.ForeignKey(continent_cls.id))
-        continent = sa.orm.relationship(
+        id: Mapped[int] = mapped_column(sa.Integer, primary_key=True)
+        name: Mapped[str] = mapped_column(sa.String(256), nullable=False)
+        continent_id: Mapped[int] = mapped_column(
+            sa.Integer, sa.ForeignKey(continent_cls.id)
+        )
+        continent: Mapped[continent_cls] = sa.orm.relationship(
             continent_cls,
             backref=sa.orm.backref("continents"),
         )
@@ -128,8 +161,8 @@ def continent_cls(base):
     class Continent(base):
         __tablename__ = "continent"
         __table_args__ = (UniqueConstraint("name"),)
-        id = sa.Column(sa.Integer, primary_key=True)
-        name = sa.Column(sa.String, nullable=False)
+        id: Mapped[int] = mapped_column(sa.Integer, primary_key=True)
+        name: Mapped[str] = mapped_column(sa.String(256), nullable=False)
 
     return Continent
 
@@ -139,8 +172,8 @@ def publisher_cls(base):
     class Publisher(base):
         __tablename__ = "publisher"
         __table_args__ = (UniqueConstraint("name"),)
-        id = sa.Column(sa.Integer, primary_key=True)
-        name = sa.Column(sa.String, nullable=False)
+        id: Mapped[int] = mapped_column(sa.Integer, primary_key=True)
+        name: Mapped[str] = mapped_column(sa.String(256), nullable=False)
 
     return Publisher
 
@@ -150,11 +183,15 @@ def author_cls(base, city_cls):
     class Author(base):
         __tablename__ = "author"
         __table_args__ = (UniqueConstraint("name"),)
-        id = sa.Column(sa.Integer, primary_key=True)
-        name = sa.Column(sa.String, nullable=False)
-        birth_year = sa.Column(sa.Integer, nullable=True)
-        city_id = sa.Column(sa.Integer, sa.ForeignKey(city_cls.id))
-        city = sa.orm.relationship(city_cls, backref=sa.orm.backref("city"))
+        id: Mapped[int] = mapped_column(sa.Integer, primary_key=True)
+        name: Mapped[str] = mapped_column(sa.String(256), nullable=False)
+        birth_year: Mapped[int] = mapped_column(sa.Integer, nullable=True)
+        city_id: Mapped[int] = mapped_column(
+            sa.Integer, sa.ForeignKey(city_cls.id)
+        )
+        city: Mapped[city_cls] = sa.orm.relationship(
+            city_cls, backref=sa.orm.backref("city")
+        )
 
     return Author
 
@@ -164,8 +201,8 @@ def shelf_cls(base):
     class Shelf(base):
         __tablename__ = "shelf"
         __table_args__ = (UniqueConstraint("shelf"),)
-        id = sa.Column(sa.Integer, primary_key=True)
-        shelf = sa.Column(sa.String, nullable=False)
+        id: Mapped[int] = mapped_column(sa.Integer, primary_key=True)
+        shelf: Mapped[str] = mapped_column(sa.String(256), nullable=False)
 
     return Shelf
 
@@ -175,8 +212,8 @@ def subject_cls(base):
     class Subject(base):
         __tablename__ = "subject"
         __table_args__ = (UniqueConstraint("name"),)
-        id = sa.Column(sa.Integer, primary_key=True)
-        name = sa.Column(sa.String, nullable=False)
+        id: Mapped[int] = mapped_column(sa.Integer, primary_key=True)
+        name: Mapped[str] = mapped_column(sa.String(256), nullable=False)
 
     return Subject
 
@@ -186,8 +223,8 @@ def language_cls(base):
     class Language(base):
         __tablename__ = "language"
         __table_args__ = (UniqueConstraint("code"),)
-        id = sa.Column(sa.Integer, primary_key=True)
-        code = sa.Column(sa.String, nullable=False)
+        id: Mapped[int] = mapped_column(sa.Integer, primary_key=True)
+        code: Mapped[str] = mapped_column(sa.String(256), nullable=False)
 
     return Language
 
@@ -197,8 +234,8 @@ def contact_cls(base):
     class Contact(base):
         __tablename__ = "contact"
         __table_args__ = (UniqueConstraint("name"),)
-        id = sa.Column(sa.Integer, primary_key=True)
-        name = sa.Column(sa.String, nullable=False)
+        id: Mapped[int] = mapped_column(sa.Integer, primary_key=True)
+        name: Mapped[str] = mapped_column(sa.String(256), nullable=False)
 
     return Contact
 
@@ -211,10 +248,12 @@ def contact_item_cls(base, contact_cls):
             UniqueConstraint("name"),
             UniqueConstraint("contact_id"),
         )
-        id = sa.Column(sa.Integer, primary_key=True)
-        name = sa.Column(sa.String, nullable=False)
-        contact_id = sa.Column(sa.Integer, sa.ForeignKey(contact_cls.id))
-        contact = sa.orm.relationship(
+        id: Mapped[int] = mapped_column(sa.Integer, primary_key=True)
+        name: Mapped[str] = mapped_column(sa.String(256), nullable=False)
+        contact_id: Mapped[int] = mapped_column(
+            sa.Integer, sa.ForeignKey(contact_cls.id)
+        )
+        contact: Mapped[contact_cls] = sa.orm.relationship(
             contact_cls, backref=sa.orm.backref("contacts")
         )
 
@@ -229,10 +268,14 @@ def user_cls(base, contact_cls):
             UniqueConstraint("name"),
             UniqueConstraint("contact_id"),
         )
-        id = sa.Column(sa.Integer, primary_key=True, autoincrement=True)
-        name = sa.Column(sa.String, nullable=False)
-        contact_id = sa.Column(sa.Integer, sa.ForeignKey(contact_cls.id))
-        contact = sa.orm.relationship(
+        id: Mapped[int] = mapped_column(
+            sa.Integer, primary_key=True, autoincrement=True
+        )
+        name: Mapped[str] = mapped_column(sa.String(256), nullable=False)
+        contact_id: Mapped[int] = mapped_column(
+            sa.Integer, sa.ForeignKey(contact_cls.id)
+        )
+        contact: Mapped[contact_cls] = sa.orm.relationship(
             contact_cls, backref=sa.orm.backref("user_contacts")
         )
 
@@ -244,25 +287,33 @@ def book_cls(base, publisher_cls, user_cls):
     class Book(base):
         __tablename__ = "book"
         __table_args__ = (UniqueConstraint("isbn"),)
-        isbn = sa.Column(sa.String, primary_key=True)
-        title = sa.Column(sa.String, nullable=False)
-        description = sa.Column(sa.String, nullable=True)
-        copyright = sa.Column(sa.String, nullable=True)
-        publisher_id = sa.Column(sa.Integer, sa.ForeignKey(publisher_cls.id))
-        publisher = sa.orm.relationship(
+        isbn: Mapped[str] = mapped_column(sa.String(256), primary_key=True)
+        title: Mapped[str] = mapped_column(sa.String(256), nullable=False)
+        description: Mapped[str] = mapped_column(sa.String(256), nullable=True)
+        copyright: Mapped[str] = mapped_column(sa.String(256), nullable=True)
+        publisher_id: Mapped[int] = mapped_column(
+            sa.Integer, sa.ForeignKey(publisher_cls.id), nullable=True
+        )
+        publisher: Mapped[publisher_cls] = sa.orm.relationship(
             publisher_cls, backref=sa.orm.backref("publishers")
         )
-        buyer_id = sa.Column(sa.Integer, sa.ForeignKey(user_cls.id))
-        buyer = sa.orm.relationship(
-            user_cls, backref=sa.orm.backref("buyers"), foreign_keys=[buyer_id]
+        buyer_id: Mapped[int] = mapped_column(
+            sa.Integer, sa.ForeignKey(user_cls.id), nullable=True
         )
-        seller_id = sa.Column(sa.Integer, sa.ForeignKey(user_cls.id))
-        seller = sa.orm.relationship(
+        buyer: Mapped[user_cls] = sa.orm.relationship(
+            user_cls,
+            backref=sa.orm.backref("buyers"),
+            foreign_keys=[buyer_id],
+        )
+        seller_id: Mapped[int] = mapped_column(
+            sa.Integer, sa.ForeignKey(user_cls.id), nullable=True
+        )
+        seller: Mapped[user_cls] = sa.orm.relationship(
             user_cls,
             backref=sa.orm.backref("sellers"),
             foreign_keys=[seller_id],
         )
-        tags = sa.Column(sa.dialects.postgresql.JSONB, nullable=True)
+        tags: Mapped[sa.JSON] = mapped_column(sa.JSON, nullable=True)
 
     return Book
 
@@ -272,13 +323,17 @@ def book_author_cls(base, book_cls, author_cls):
     class BookAuthor(base):
         __tablename__ = "book_author"
         __table_args__ = (UniqueConstraint("book_isbn", "author_id"),)
-        id = sa.Column(sa.Integer, primary_key=True)
-        book_isbn = sa.Column(sa.String, sa.ForeignKey(book_cls.isbn))
-        book = sa.orm.relationship(
+        id: Mapped[int] = mapped_column(sa.Integer, primary_key=True)
+        book_isbn: Mapped[str] = mapped_column(
+            sa.String(256), sa.ForeignKey(book_cls.isbn)
+        )
+        book: Mapped[book_cls] = sa.orm.relationship(
             book_cls, backref=sa.orm.backref("book_author_books")
         )
-        author_id = sa.Column(sa.Integer, sa.ForeignKey(author_cls.id))
-        author = sa.orm.relationship(
+        author_id: Mapped[int] = mapped_column(
+            sa.Integer, sa.ForeignKey(author_cls.id)
+        )
+        author: Mapped[author_cls] = sa.orm.relationship(
             author_cls, backref=sa.orm.backref("authors")
         )
 
@@ -290,13 +345,17 @@ def book_subject_cls(base, book_cls, subject_cls):
     class BookSubject(base):
         __tablename__ = "book_subject"
         __table_args__ = (UniqueConstraint("book_isbn", "subject_id"),)
-        id = sa.Column(sa.Integer, primary_key=True)
-        book_isbn = sa.Column(sa.String, sa.ForeignKey(book_cls.isbn))
-        book = sa.orm.relationship(
+        id: Mapped[int] = mapped_column(sa.Integer, primary_key=True)
+        book_isbn: Mapped[str] = mapped_column(
+            sa.String(256), sa.ForeignKey(book_cls.isbn)
+        )
+        book: Mapped[book_cls] = sa.orm.relationship(
             book_cls, backref=sa.orm.backref("book_subject_books")
         )
-        subject_id = sa.Column(sa.Integer, sa.ForeignKey(subject_cls.id))
-        subject = sa.orm.relationship(
+        subject_id: Mapped[int] = mapped_column(
+            sa.Integer, sa.ForeignKey(subject_cls.id)
+        )
+        subject: Mapped[subject_cls] = sa.orm.relationship(
             subject_cls, backref=sa.orm.backref("subjects")
         )
 
@@ -308,13 +367,17 @@ def book_language_cls(base, book_cls, language_cls):
     class BookLanguage(base):
         __tablename__ = "book_language"
         __table_args__ = (UniqueConstraint("book_isbn", "language_id"),)
-        id = sa.Column(sa.Integer, primary_key=True)
-        book_isbn = sa.Column(sa.String, sa.ForeignKey(book_cls.isbn))
-        book = sa.orm.relationship(
+        id: Mapped[int] = mapped_column(sa.Integer, primary_key=True)
+        book_isbn: Mapped[str] = mapped_column(
+            sa.String(256), sa.ForeignKey(book_cls.isbn)
+        )
+        book: Mapped[book_cls] = sa.orm.relationship(
             book_cls, backref=sa.orm.backref("book_language_books")
         )
-        language_id = sa.Column(sa.Integer, sa.ForeignKey(language_cls.id))
-        language = sa.orm.relationship(
+        language_id: Mapped[int] = mapped_column(
+            sa.Integer, sa.ForeignKey(language_cls.id)
+        )
+        language: Mapped[language_cls] = sa.orm.relationship(
             language_cls, backref=sa.orm.backref("languages")
         )
 
@@ -326,13 +389,17 @@ def book_shelf_cls(base, book_cls, shelf_cls):
     class BookShelf(base):
         __tablename__ = "book_shelf"
         __table_args__ = (UniqueConstraint("book_isbn", "shelf_id"),)
-        id = sa.Column(sa.Integer, primary_key=True)
-        book_isbn = sa.Column(sa.String, sa.ForeignKey(book_cls.isbn))
-        book = sa.orm.relationship(
+        id: Mapped[int] = mapped_column(sa.Integer, primary_key=True)
+        book_isbn: Mapped[str] = mapped_column(
+            sa.String(256), sa.ForeignKey(book_cls.isbn)
+        )
+        book: Mapped[book_cls] = sa.orm.relationship(
             book_cls, backref=sa.orm.backref("book_book_shelf_books")
         )
-        shelf_id = sa.Column(sa.Integer, sa.ForeignKey(shelf_cls.id))
-        shelf = sa.orm.relationship(
+        shelf_id: Mapped[int] = mapped_column(
+            sa.Integer, sa.ForeignKey(shelf_cls.id)
+        )
+        shelf: Mapped[shelf_cls] = sa.orm.relationship(
             shelf_cls, backref=sa.orm.backref("shelves")
         )
 
@@ -344,12 +411,14 @@ def rating_cls(base, book_cls):
     class Rating(base):
         __tablename__ = "rating"
         __table_args__ = (UniqueConstraint("book_isbn"),)
-        id = sa.Column(sa.Integer, primary_key=True)
-        book_isbn = sa.Column(sa.String, sa.ForeignKey(book_cls.isbn))
-        book = sa.orm.relationship(
+        id: Mapped[int] = mapped_column(sa.Integer, primary_key=True)
+        book_isbn: Mapped[str] = mapped_column(
+            sa.String(256), sa.ForeignKey(book_cls.isbn)
+        )
+        book: Mapped[book_cls] = sa.orm.relationship(
             book_cls, backref=sa.orm.backref("book_rating_books")
         )
-        value = sa.Column(sa.Float, nullable=True)
+        value: Mapped[float] = mapped_column(sa.Float, nullable=True)
 
     return Rating
 
@@ -359,8 +428,8 @@ def group_cls(base):
     class Group(base):
         __tablename__ = "group"
         __table_args__ = (UniqueConstraint("group_name"),)
-        id = sa.Column(sa.Integer, primary_key=True)
-        group_name = sa.Column(sa.String, nullable=False)
+        id: Mapped[int] = mapped_column(sa.Integer, primary_key=True)
+        group_name: Mapped[str] = mapped_column(sa.String(256), nullable=False)
 
     return Group
 
@@ -370,13 +439,17 @@ def book_group_cls(base, book_cls, group_cls):
     class BookGroup(base):
         __tablename__ = "book_group"
         __table_args__ = (UniqueConstraint("book_isbn", "group_id"),)
-        id = sa.Column(sa.Integer, primary_key=True)
-        book_isbn = sa.Column(sa.String, sa.ForeignKey(book_cls.isbn))
-        book = sa.orm.relationship(
+        id: Mapped[int] = mapped_column(sa.Integer, primary_key=True)
+        book_isbn: Mapped[str] = mapped_column(
+            sa.String(256), sa.ForeignKey(book_cls.isbn)
+        )
+        book: Mapped[book_cls] = sa.orm.relationship(
             book_cls, backref=sa.orm.backref("book_book_group_books")
         )
-        group_id = sa.Column(sa.Integer, sa.ForeignKey(group_cls.id))
-        group = sa.orm.relationship(
+        group_id: Mapped[int] = mapped_column(
+            sa.Integer, sa.ForeignKey(group_cls.id)
+        )
+        group: Mapped[group_cls] = sa.orm.relationship(
             group_cls, backref=sa.orm.backref("groups"), cascade="all,delete"
         )
 
@@ -431,21 +504,41 @@ def model_mapping(
 @pytest.fixture(scope="session")
 def table_creator(base, connection, model_mapping):
     sa.orm.configure_mappers()
-    base.metadata.create_all(connection)
+    with connection.engine.connect() as conn:
+        base.metadata.create_all(connection.engine)
+        conn.commit()
     pg_base = Base(connection.engine.url.database)
-    pg_base.create_triggers(
-        connection.engine.url.database,
-        DEFAULT_SCHEMA,
-    )
-    pg_base.drop_replication_slot(f"{connection.engine.url.database}_testdb")
-    pg_base.create_replication_slot(f"{connection.engine.url.database}_testdb")
+    if not IS_MYSQL_COMPAT:
+        pg_base.create_triggers(
+            connection.engine.url.database,
+            DEFAULT_SCHEMA,
+        )
+        pg_base.drop_replication_slot(
+            f"{connection.engine.url.database}_testdb"
+        )
+        pg_base.create_replication_slot(
+            f"{connection.engine.url.database}_testdb"
+        )
     yield
-    pg_base.drop_replication_slot(f"{connection.engine.url.database}_testdb")
-    base.metadata.drop_all(connection)
+    if not IS_MYSQL_COMPAT:
+        try:
+            pg_base.drop_replication_slot(
+                f"{connection.engine.url.database}_testdb"
+            )
+        except Exception:
+            pass
+    with connection.engine.connect() as conn:
+        base.metadata.drop_all(connection.engine)
+        conn.commit()
     try:
         os.unlink(f".{connection.engine.url.database}_testdb")
     except (OSError, FileNotFoundError):
         pass
+    # Cleanup pg_base connections
+    try:
+        pg_base.engine.dispose(close=True)
+    except Exception:
+        pass
 
 
 @pytest.fixture(scope="session")
@@ -468,7 +561,6 @@ def dataset(
     book_group_cls,
     group_cls,
 ):
-
     eu_continent = continent_cls(name="Europe")
     na_continent = continent_cls(name="North America")
     session.add(eu_continent)
