@@ -1,4 +1,5 @@
 """Constants tests."""
+
 from pgsync.constants import (
     ELASTICSEARCH_MAPPING_PARAMETERS,
     ELASTICSEARCH_TYPES,
@@ -94,9 +95,11 @@ class TestConstants(object):
                 "binary",
                 "boolean",
                 "byte",
+                "completion",
                 "constant_keyword",
                 "date",
                 "double",
+                "dense_vector",
                 "float",
                 "geo_point",
                 "geo_shape",
@@ -145,6 +148,7 @@ class TestConstants(object):
                 "coerce",
                 "copy_to",
                 "doc_values",
+                "dimension",
                 "dynamic",
                 "eager_global_ordinals",
                 "enabled",
