@@ -1,4 +1,5 @@
 """Node tests."""
+
 import pytest
 
 from pgsync.base import Base
@@ -12,12 +13,17 @@ from pgsync.exc import (
     TableNotInNodeError,
 )
 from pgsync.node import ForeignKey, Node, Relationship, Tree
+from pgsync.settings import IS_MYSQL_COMPAT
 
 
 @pytest.mark.usefixtures("table_creator")
 class TestNode(object):
     """Node tests."""
 
+    @classmethod
+    def setup_class(cls):
+        cls.schema = "testdb" if IS_MYSQL_COMPAT else "public"
+
     @pytest.fixture(scope="function")
     def nodes(self):
         return {
@@ -115,13 +121,13 @@ class TestNode(object):
         node = Node(
             models=pg_base.models,
             table="book",
-            schema="public",
+            schema=self.schema,
             label="book_label",
         )
-        assert str(node) == "Node: public.book_label"
+        assert str(node) == f"Node: {self.schema}.book_label"
 
     def test_traverse_breadth_first(self, sync, nodes):
-        root = Tree(sync.models).build(nodes)
+        root = Tree(sync.models, nodes=nodes, database=self.schema)
         root.display()
         for i, node in enumerate(root.traverse_breadth_first()):
             if i == 0:
@@ -145,7 +151,7 @@ class TestNode(object):
         sync.search_client.close()
 
     def test_traverse_post_order(self, sync, nodes):
-        root = Tree(sync.models).build(nodes)
+        root: Tree = Tree(sync.models, nodes=nodes, database=self.schema)
         root.display()
         for i, node in enumerate(root.traverse_post_order()):
             if i == 0:
@@ -182,7 +188,7 @@ class TestNode(object):
             ],
         }
         with pytest.raises(RelationshipAttributeError) as excinfo:
-            Tree(sync.models).build(nodes)
+            Tree(sync.models, nodes=nodes, database=self.schema)
         assert "Relationship attribute " in str(excinfo.value)
         sync.search_client.close()
 
@@ -199,38 +205,41 @@ class TestNode(object):
                 },
             ],
         }
-        tree = Tree(sync.models)
-        tree.build(nodes)
-        node = tree.get_node("book", "public")
-        assert str(node) == "Node: public.book"
+        tree: Tree = Tree(sync.models, nodes=nodes, database=self.schema)
+        node = tree.get_node("book", self.schema)
+        assert str(node) == f"Node: {self.schema}.book"
 
         with pytest.raises(RuntimeError) as excinfo:
-            tree.get_node("xxx", "public")
-        assert "Node for public.xxx not found" in str(excinfo.value)
+            tree.get_node("xxx", self.schema)
+        assert f"Node for {self.schema}.xxx not found" in str(excinfo.value)
 
         sync.search_client.close()
 
     def test_tree_build(self, sync):
-
         with pytest.raises(TableNotInNodeError) as excinfo:
-            Tree(sync.models).build(
-                {
+            Tree(
+                sync.models,
+                nodes={
                     "table": None,
-                }
+                },
+                database=self.schema,
             )
 
         with pytest.raises(NodeAttributeError) as excinfo:
-            Tree(sync.models).build(
-                {
+            Tree(
+                sync.models,
+                nodes={
                     "table": "book",
                     "foo": "bar",
-                }
+                },
+                database=self.schema,
             )
         assert "Unknown node attribute(s):" in str(excinfo.value)
 
         with pytest.raises(NodeAttributeError) as excinfo:
-            Tree(sync.models).build(
-                {
+            Tree(
+                sync.models,
+                nodes={
                     "table": "book",
                     "children": [
                         {
@@ -243,13 +252,15 @@ class TestNode(object):
                             "foo": "bar",
                         },
                     ],
-                }
+                },
+                database=self.schema,
             )
         assert "Unknown node attribute(s):" in str(excinfo.value)
 
         with pytest.raises(TableNotInNodeError) as excinfo:
-            Tree(sync.models).build(
-                {
+            Tree(
+                sync.models,
+                nodes={
                     "table": "book",
                     "children": [
                         {
@@ -260,15 +271,18 @@ class TestNode(object):
                             },
                         },
                     ],
-                }
+                },
+                database=self.schema,
             )
         assert "Table not specified in node" in str(excinfo.value)
 
-        Tree(sync.models).build(
-            {
+        Tree(
+            sync.models,
+            nodes={
                 "table": "book",
                 "columns": ["tags->0"],
-            }
+            },
+            database=self.schema,
         )
 
         sync.search_client.close()
