@@ -1,15 +1,25 @@
 """QueryBuilder tests."""
+
 import pytest
 
 from pgsync.base import Base
 from pgsync.node import Node
 from pgsync.querybuilder import QueryBuilder
+from pgsync.settings import IS_MYSQL_COMPAT
 
 
 @pytest.mark.usefixtures("table_creator")
 class TestQueryBuilder(object):
     """QueryBuilder tests."""
 
+    @classmethod
+    def setup_class(cls):
+        cls.schema = "testdb" if IS_MYSQL_COMPAT else "public"
+
+    @pytest.mark.skipif(
+        IS_MYSQL_COMPAT,
+        reason="Skipped because IS_MYSQL_COMPAT env var is set",
+    )
     def test__json_build_object(self, connection):
         pg_base = Base(connection.engine.url.database)
         query_builder = QueryBuilder()
@@ -20,7 +30,7 @@ class TestQueryBuilder(object):
         node = Node(
             models=pg_base.models,
             table="book",
-            schema="public",
+            schema=self.schema,
         )
         expression = query_builder._json_build_object(node.columns)
         assert expression is not None
@@ -38,17 +48,14 @@ class TestQueryBuilder(object):
         )
         assert expression is not None
         expected = (
-            "CAST(JSON_BUILD_OBJECT(:JSON_BUILD_OBJECT_1, book_1.isbn) AS "
-            "JSONB) || CAST(JSON_BUILD_OBJECT(:JSON_BUILD_OBJECT_2, "
-            "book_1.title) AS JSONB) || CAST(JSON_BUILD_OBJECT(:"
-            "JSON_BUILD_OBJECT_3, book_1.description) AS JSONB) || "
-            "CAST(JSON_BUILD_OBJECT(:JSON_BUILD_OBJECT_4, book_1.copyright) "
-            "AS JSONB) || CAST(JSON_BUILD_OBJECT(:JSON_BUILD_OBJECT_5, "
-            "book_1.publisher_id) AS JSONB) || CAST(JSON_BUILD_OBJECT("
-            ":JSON_BUILD_OBJECT_6, book_1.buyer_id) AS JSONB) || "
-            "CAST(JSON_BUILD_OBJECT(:JSON_BUILD_OBJECT_7, book_1.seller_id) "
-            "AS JSONB) || CAST(JSON_BUILD_OBJECT(:JSON_BUILD_OBJECT_8, "
-            "book_1.tags) AS JSONB)"
+            "((((((CAST(JSON_BUILD_OBJECT(:JSON_BUILD_OBJECT_1, book_1.isbn) AS JSONB) || "
+            "CAST(JSON_BUILD_OBJECT(:JSON_BUILD_OBJECT_2, book_1.title) AS JSONB)) || "
+            "CAST(JSON_BUILD_OBJECT(:JSON_BUILD_OBJECT_3, book_1.description) AS JSONB)) || "
+            "CAST(JSON_BUILD_OBJECT(:JSON_BUILD_OBJECT_4, book_1.copyright) AS JSONB)) || "
+            "CAST(JSON_BUILD_OBJECT(:JSON_BUILD_OBJECT_5, book_1.publisher_id) AS JSONB)) || "
+            "CAST(JSON_BUILD_OBJECT(:JSON_BUILD_OBJECT_6, book_1.buyer_id) AS JSONB)) || "
+            "CAST(JSON_BUILD_OBJECT(:JSON_BUILD_OBJECT_7, book_1.seller_id) AS JSONB)) || "
+            "CAST(JSON_BUILD_OBJECT(:JSON_BUILD_OBJECT_8, book_1.tags) AS JSONB)"
         )
         assert str(expression) == expected
 
@@ -57,14 +64,14 @@ class TestQueryBuilder(object):
         query_builder = QueryBuilder()
 
         foreign_keys = {
-            "public.subject": ["column_a", "column_b", "column_X"],
-            "schema.table_b": ["column_x"],
+            f"{self.schema}.subject": ["column_a", "column_b", "column_X"],
+            f"{self.schema}.table_b": ["column_x"],
         }
 
         subject = Node(
             models=pg_base.models,
             table="subject",
-            schema="public",
+            schema=self.schema,
             relationship={
                 "type": "one_to_many",
                 "variant": "scalar",
