@@ -5,13 +5,18 @@ import psycopg2
 import pytest
 
 from pgsync.base import subtransactions
-from pgsync.settings import NTHREADS_POLLDB
+from pgsync.node import Tree
+from pgsync.settings import IS_MYSQL_COMPAT
 from pgsync.singleton import Singleton
 from pgsync.sync import Sync
 
 from .testing_utils import assert_resync_empty, noop, search, sort_list
 
 
+@pytest.mark.skipif(
+    IS_MYSQL_COMPAT,
+    reason="Skipped because IS_MYSQL_COMPAT env var is set",
+)
 @pytest.mark.usefixtures("table_creator")
 class TestNestedChildren(object):
     """Root and nested childred node tests."""
@@ -213,9 +218,6 @@ class TestNestedChildren(object):
 
         with subtransactions(session):
             conn = session.connection().engine.connect().connection
-            conn.set_isolation_level(
-                psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT
-            )
             cursor = conn.cursor()
             channel = sync.database
             cursor.execute(f"UNLISTEN {channel}")
@@ -286,8 +288,7 @@ class TestNestedChildren(object):
             raise
 
         sync.redis.delete()
-        session.connection().engine.connect().close()
-        session.connection().engine.dispose()
+        session.connection().engine.dispose(close=True)
         sync.search_client.close()
 
     @pytest.fixture(scope="function")
@@ -384,9 +385,7 @@ class TestNestedChildren(object):
 
     def test_sync(self, sync, nodes, data):
         """Test regular sync produces the correct result."""
-        sync.tree.__post_init__()
-        sync.nodes = nodes
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         assert len(docs) == 3
         docs = sorted(docs, key=lambda k: k["_id"])
@@ -396,6 +395,7 @@ class TestNestedChildren(object):
                 "_index": "testdb",
                 "_source": {
                     "_meta": {
+                        "book": {"isbn": ["abc"]},
                         "author": {"id": [1, 4]},
                         "book_author": {"id": [1, 4]},
                         "book_language": {"id": [1, 4, 7, 9]},
@@ -454,6 +454,7 @@ class TestNestedChildren(object):
                 "_index": "testdb",
                 "_source": {
                     "_meta": {
+                        "book": {"isbn": ["def"]},
                         "author": {"id": [1, 2]},
                         "book_author": {"id": [2, 5]},
                         "book_language": {"id": [2, 5, 8]},
@@ -511,6 +512,7 @@ class TestNestedChildren(object):
                 "_index": "testdb",
                 "_source": {
                     "_meta": {
+                        "book": {"isbn": ["ghi"]},
                         "author": {"id": [2, 3]},
                         "book_author": {"id": [3, 6]},
                         "book_language": {"id": [3, 6]},
@@ -679,14 +681,14 @@ class TestNestedChildren(object):
             book_shelf_cls(id=7, book=books[0], shelf=shelves[1]),
         ]
 
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": nodes,
         }
 
-        # 1. sync first to add the initial document
-        sync = Sync(document)
+        # 1. sync first to add the initial doc
+        sync = Sync(doc)
         sync.search_client.bulk(
             sync.index, [sort_list(doc) for doc in sync.sync()]
         )
@@ -713,6 +715,7 @@ class TestNestedChildren(object):
                 "_index": "testdb",
                 "_source": {
                     "_meta": {
+                        "book": {"isbn": ["jkl"]},
                         "author": {"id": [5]},
                         "book_author": {"id": [7]},
                         "book_language": {"id": [10, 11]},
@@ -758,6 +761,7 @@ class TestNestedChildren(object):
                 "_index": "testdb",
                 "_source": {
                     "_meta": {
+                        "book": {"isbn": ["mno"]},
                         "author": {"id": [5]},
                         "book_author": {"id": [8]},
                         "book_language": {"id": [12, 13]},
@@ -804,15 +808,18 @@ class TestNestedChildren(object):
         ]
         assert_resync_empty(sync, nodes)
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_update_root(self, data, nodes, book_cls):
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": nodes,
         }
-        # 1. sync first to add the initial document
-        sync = Sync(document)
+        # 1. sync first to add the initial doc
+        sync = Sync(doc)
         sync.search_client.bulk(
             sync.index, [sort_list(doc) for doc in sync.sync()]
         )
@@ -839,6 +846,7 @@ class TestNestedChildren(object):
                 "_index": "testdb",
                 "_source": {
                     "_meta": {
+                        "book": {"isbn": ["abc"]},
                         "author": {"id": [1, 4]},
                         "book_author": {"id": [1, 4]},
                         "book_language": {"id": [1, 4, 7, 9]},
@@ -918,6 +926,9 @@ class TestNestedChildren(object):
 
         assert_resync_empty(sync, nodes)
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_delete_root(
         self,
@@ -929,13 +940,13 @@ class TestNestedChildren(object):
         book_subject_cls,
         book_author_cls,
     ):
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": nodes,
         }
-        # 1. sync first to add the initial document
-        sync = Sync(document)
+        # 1. sync first to add the initial doc
+        sync = Sync(doc)
         sync.search_client.bulk(
             sync.index, [sort_list(doc) for doc in sync.sync()]
         )
@@ -991,11 +1002,11 @@ class TestNestedChildren(object):
                             "pgsync.sync.Sync.status",
                             side_effect=noop,
                         ):
-                            sync.receive(NTHREADS_POLLDB)
+                            sync.receive()
                             sync.search_client.refresh("testdb")
 
         txmin = sync.checkpoint
-        sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync(txmin=txmin)]
         assert len(docs) == 0
 
@@ -1006,6 +1017,7 @@ class TestNestedChildren(object):
         expected = [
             {
                 "_meta": {
+                    "book": {"isbn": ["def"]},
                     "author": {"id": [1, 2]},
                     "book_author": {"id": [2, 5]},
                     "book_language": {"id": [2, 5, 8]},
@@ -1059,6 +1071,7 @@ class TestNestedChildren(object):
             },
             {
                 "_meta": {
+                    "book": {"isbn": ["ghi"]},
                     "author": {"id": [2, 3]},
                     "book_author": {"id": [3, 6]},
                     "book_language": {"id": [3, 6]},
@@ -1131,6 +1144,9 @@ class TestNestedChildren(object):
 
         assert_resync_empty(sync, nodes)
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_insert_through_child_op2(
         self, book_cls, group_cls, book_group_cls, data
@@ -1151,14 +1167,14 @@ class TestNestedChildren(object):
                 }
             ],
         }
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": nodes,
         }
 
-        sync = Sync(document)
-        sync.tree.build(nodes)
+        sync = Sync(doc)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         session = sync.session
 
         with subtransactions(session):
@@ -1179,7 +1195,7 @@ class TestNestedChildren(object):
                     "isbn": "abc",
                     "group": None,
                     "title": "The Tiger Club",
-                    "_meta": {},
+                    "_meta": {"book": {"isbn": ["abc"]}},
                 },
             },
             {
@@ -1189,7 +1205,7 @@ class TestNestedChildren(object):
                     "isbn": "def",
                     "group": None,
                     "title": "The Lion Club",
-                    "_meta": {},
+                    "_meta": {"book": {"isbn": ["def"]}},
                 },
             },
             {
@@ -1199,7 +1215,7 @@ class TestNestedChildren(object):
                     "isbn": "ghi",
                     "group": None,
                     "title": "The Rabbit Club",
-                    "_meta": {},
+                    "_meta": {"book": {"isbn": ["ghi"]}},
                 },
             },
         ]
@@ -1237,7 +1253,7 @@ class TestNestedChildren(object):
                             "pgsync.sync.Sync.status",
                             side_effect=noop,
                         ):
-                            sync.receive(NTHREADS_POLLDB)
+                            sync.receive()
                             sync.search_client.refresh("testdb")
 
         docs = [sort_list(doc) for doc in sync.sync()]
@@ -1254,6 +1270,7 @@ class TestNestedChildren(object):
                     ],
                     "title": "The Tiger Club",
                     "_meta": {
+                        "book": {"isbn": ["abc"]},
                         "group": {"id": [1, 2]},
                         "book_group": {"id": [1, 2]},
                     },
@@ -1266,7 +1283,7 @@ class TestNestedChildren(object):
                     "isbn": "def",
                     "group": None,
                     "title": "The Lion Club",
-                    "_meta": {},
+                    "_meta": {"book": {"isbn": ["def"]}},
                 },
             },
             {
@@ -1276,11 +1293,14 @@ class TestNestedChildren(object):
                     "isbn": "ghi",
                     "group": None,
                     "title": "The Rabbit Club",
-                    "_meta": {},
+                    "_meta": {"book": {"isbn": ["ghi"]}},
                 },
             },
         ]
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_update_through_child_noop(self, sync, data):
         # update a new through child with noop
@@ -1324,14 +1344,14 @@ class TestNestedChildren(object):
             ),
         )
 
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": nodes,
         }
 
-        # 1. sync first to add the initial document
-        sync = Sync(document)
+        # 1. sync first to add the initial doc
+        sync = Sync(doc)
 
         session = sync.session
 
@@ -1351,6 +1371,7 @@ class TestNestedChildren(object):
         expected = [
             {
                 "_meta": {
+                    "book": {"isbn": ["abc"]},
                     "author": {"id": [1, 4, 5]},
                     "book_author": {"id": [1, 4, 7]},
                     "book_language": {"id": [1, 4, 7, 9]},
@@ -1418,6 +1439,7 @@ class TestNestedChildren(object):
             },
             {
                 "_meta": {
+                    "book": {"isbn": ["def"]},
                     "author": {"id": [1, 2]},
                     "book_author": {"id": [2, 5]},
                     "book_language": {"id": [2, 5, 8]},
@@ -1471,6 +1493,7 @@ class TestNestedChildren(object):
             },
             {
                 "_meta": {
+                    "book": {"isbn": ["ghi"]},
                     "author": {"id": [2, 3]},
                     "book_author": {"id": [3, 6]},
                     "book_language": {"id": [3, 6]},
@@ -1557,14 +1580,14 @@ class TestNestedChildren(object):
         continent_cls,
     ):
         # update a new through child with op
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": nodes,
         }
 
-        # 1. sync first to add the initial document
-        sync = Sync(document)
+        # 1. sync first to add the initial doc
+        sync = Sync(doc)
         sync.search_client.bulk(
             sync.index, [sort_list(doc) for doc in sync.sync()]
         )
@@ -1607,6 +1630,7 @@ class TestNestedChildren(object):
         expected = [
             {
                 "_meta": {
+                    "book": {"isbn": ["abc"]},
                     "author": {"id": [4, 5]},
                     "book_author": {"id": [1, 4]},
                     "book_language": {"id": [1, 4, 7, 9]},
@@ -1661,6 +1685,7 @@ class TestNestedChildren(object):
             },
             {
                 "_meta": {
+                    "book": {"isbn": ["def"]},
                     "author": {"id": [1, 2]},
                     "book_author": {"id": [2, 5]},
                     "book_language": {"id": [2, 5, 8]},
@@ -1714,6 +1739,7 @@ class TestNestedChildren(object):
             },
             {
                 "_meta": {
+                    "book": {"isbn": ["ghi"]},
                     "author": {"id": [2, 3]},
                     "book_author": {"id": [3, 6]},
                     "book_language": {"id": [3, 6]},
@@ -1786,17 +1812,125 @@ class TestNestedChildren(object):
 
         assert_resync_empty(sync, nodes)
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
-    def test_delete_through_child_op(self, sync, data, nodes, book_author_cls):
-        # delete a new through child with op
-        document = {
+    def test_insert_grand_child_through_child_op(
+        self,
+        data,
+        nodes,
+        book_cls,
+        author_cls,
+        city_cls,
+        country_cls,
+        continent_cls,
+        book_author_cls,
+    ):
+        """
+        Insert a new grand child (author) via the through child (book_author),
+        with an entirely new city/country/continent (Sydney/Australia).
+        """
+        book_author = book_author_cls(
+            id=8,
+            book_isbn="def",  # attach new author to existing book 'def'
+            author=author_cls(
+                id=6,
+                name="Italo Calvino",
+                birth_year=1923,
+                city=city_cls(
+                    id=6,
+                    name="Sydney",
+                    country=country_cls(
+                        id=6,
+                        name="Australia",
+                        continent=continent_cls(
+                            id=7,
+                            name="Australia",
+                        ),
+                    ),
+                ),
+            ),
+        )
+
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": nodes,
         }
 
-        # 1. sync first to add the initial document
-        sync = Sync(document)
+        sync = Sync(doc)
+        session = sync.session
+        with subtransactions(session):
+            session.add(book_author)
+
+        sync.search_client.bulk(
+            sync.index, [sort_list(doc) for doc in sync.sync()]
+        )
+        sync.search_client.refresh("testdb")
+
+        docs = search(sync.search_client, "testdb")
+        assert len(docs) == 3
+        docs = sorted(docs, key=lambda k: k["isbn"])
+
+        expected_def_meta = {
+            "book": {"isbn": ["def"]},
+            "author": {"id": [1, 2, 6]},  # new author id=6
+            "book_author": {"id": [2, 5, 8]},  # new through id=8
+            "book_language": {"id": [2, 5, 8]},
+            "book_subject": {"id": [2, 5, 7]},
+            "city": {"id": [1, 2, 6]},  # Sydney id=6 added
+            "continent": {"id": [1, 2, 7]},  # Australia id=7 added
+            "country": {"id": [1, 2, 6]},  # Australia id=6 added
+            "language": {"id": [1, 2, 3]},
+            "publisher": {"id": [2]},
+            "subject": {"id": [2, 4, 5]},
+        }
+
+        expected_new_author = {
+            "id": 6,
+            "name": "Italo Calvino",
+            "city_label": {
+                "id": 6,
+                "name": "Sydney",
+                "country_label": {
+                    "id": 6,
+                    "name": "Australia",
+                    "continent_label": {"name": "Australia"},
+                },
+            },
+        }
+
+        # pull out the 'def' book doc
+        def_doc = next(d for d in docs if d["isbn"] == "def")
+
+        # meta checks
+        for key, val in expected_def_meta.items():
+            assert def_doc["_meta"][key] == val
+
+        # author list contains new author
+        assert any(
+            a["id"] == expected_new_author["id"] for a in def_doc["authors"]
+        )
+        new_author_doc = next(a for a in def_doc["authors"] if a["id"] == 6)
+        assert new_author_doc == expected_new_author
+
+        assert_resync_empty(sync, nodes)
+        sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
+
+    def test_delete_through_child_op(self, sync, data, nodes, book_author_cls):
+        # delete a new through child with op
+        doc = {
+            "index": "testdb",
+            "database": "testdb",
+            "nodes": nodes,
+        }
+
+        # 1. sync first to add the initial doc
+        sync = Sync(doc)
         sync.search_client.bulk(
             sync.index, [sort_list(doc) for doc in sync.sync()]
         )
@@ -1823,6 +1957,7 @@ class TestNestedChildren(object):
         expected = [
             {
                 "_meta": {
+                    "book": {"isbn": ["abc"]},
                     "book_language": {"id": [1, 4, 7, 9]},
                     "book_subject": {"id": [1, 4, 6]},
                     "language": {"id": [1, 2, 3, 4]},
@@ -1845,6 +1980,7 @@ class TestNestedChildren(object):
             },
             {
                 "_meta": {
+                    "book": {"isbn": ["def"]},
                     "author": {"id": [1, 2]},
                     "book_author": {"id": [2, 5]},
                     "book_language": {"id": [2, 5, 8]},
@@ -1898,6 +2034,7 @@ class TestNestedChildren(object):
             },
             {
                 "_meta": {
+                    "book": {"isbn": ["ghi"]},
                     "author": {"id": [2, 3]},
                     "book_author": {"id": [3, 6]},
                     "book_language": {"id": [3, 6]},
@@ -1970,6 +2107,9 @@ class TestNestedChildren(object):
 
         assert_resync_empty(sync, nodes)
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_insert_nonthrough_child_noop(
         self,
@@ -1990,14 +2130,14 @@ class TestNestedChildren(object):
             ),
         )
 
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": nodes,
         }
 
-        # 1. sync first to add the initial document
-        sync = Sync(document)
+        # 1. sync first to add the initial doc
+        sync = Sync(doc)
         sync.search_client.bulk(
             sync.index, [sort_list(doc) for doc in sync.sync()]
         )
@@ -2019,17 +2159,20 @@ class TestNestedChildren(object):
 
         assert_resync_empty(sync, nodes)
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_update_nonthrough_child_noop(self, data, nodes, shelf_cls):
         # update a new non-through child with noop
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": nodes,
         }
 
-        # 1. sync first to add the initial document
-        sync = Sync(document)
+        # 1. sync first to add the initial doc
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.checkpoint = sync.txid_current
         sync.search_client.refresh("testdb")
@@ -2053,17 +2196,20 @@ class TestNestedChildren(object):
 
         assert_resync_empty(sync, nodes)
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_delete_nonthrough_child_noop(self, data, nodes, shelf_cls):
         # delete a new non-through child with noop
-        document = {
+        doc: dict = {
             "index": "testdb",
             "database": "testdb",
             "nodes": nodes,
         }
 
-        # 1. sync first to add the initial document
-        sync = Sync(document)
+        # 1. sync first to add the initial doc
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.checkpoint = sync.txid_current
         sync.checkpoint = sync.txid_current
@@ -2088,6 +2234,9 @@ class TestNestedChildren(object):
 
         assert_resync_empty(sync, nodes)
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_insert_nonthrough_child_op(self, sync, data):
         # insert a new non-through child with op
@@ -2210,13 +2359,13 @@ class TestNestedChildren(object):
             continent=continent_cls(id=6, name="Bowser Land"),
         )
 
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": nodes,
         }
-        # sync first to add the initial document
-        sync = Sync(document)
+        # sync first to add the initial doc
+        sync = Sync(doc)
         sync.search_client.bulk(
             sync.index, [sort_list(doc) for doc in sync.sync()]
         )
@@ -2247,7 +2396,7 @@ class TestNestedChildren(object):
                             "pgsync.sync.Sync.status",
                             side_effect=noop,
                         ):
-                            sync.receive(NTHREADS_POLLDB)
+                            sync.receive()
                             sync.search_client.refresh("testdb")
 
         txmin = sync.checkpoint
@@ -2257,6 +2406,9 @@ class TestNestedChildren(object):
         docs = search(sync.search_client, "testdb")
         assert_resync_empty(sync, nodes)
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_insert_deep_nested_fk_nonthrough_child_op(
         self,
