@@ -1,6 +1,6 @@
 """Tests for `pgsync` package."""
+
 import mock
-import psycopg2
 import pytest
 
 from pgsync.base import subtransactions
@@ -9,13 +9,18 @@ from pgsync.exc import (
     NodeAttributeError,
     TableNotInNodeError,
 )
-from pgsync.settings import NTHREADS_POLLDB
+from pgsync.node import Tree
+from pgsync.settings import IS_MYSQL_COMPAT
 from pgsync.singleton import Singleton
 from pgsync.sync import Sync
 
 from .testing_utils import assert_resync_empty, noop, search, sort_list
 
 
+@pytest.mark.skipif(
+    IS_MYSQL_COMPAT,
+    reason="Skipped because IS_MYSQL_COMPAT env var is set",
+)
 @pytest.mark.usefixtures("table_creator")
 class TestRoot(object):
     """Root only node tests."""
@@ -47,9 +52,6 @@ class TestRoot(object):
 
         with subtransactions(session):
             conn = session.connection().engine.connect().connection
-            conn.set_isolation_level(
-                psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT
-            )
             cursor = conn.cursor()
             channel = sync.database
             cursor.execute(f"UNLISTEN {channel}")
@@ -67,9 +69,6 @@ class TestRoot(object):
 
         with subtransactions(session):
             conn = session.connection().engine.connect().connection
-            conn.set_isolation_level(
-                psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT
-            )
             cursor = conn.cursor()
             channel = session.connection().engine.url.database
             cursor.execute(f"UNLISTEN {channel}")
@@ -90,8 +89,7 @@ class TestRoot(object):
             raise
 
         sync.redis.delete()
-        session.connection().engine.connect().close()
-        session.connection().engine.dispose()
+        session.connection().engine.dispose(close=True)
         sync.search_client.close()
 
     def test_sync2(self, sync, data):
@@ -99,15 +97,16 @@ class TestRoot(object):
         nodes = {"table": "book", "columns": ["isbn", "title", "description"]}
         txmin = sync.checkpoint
         txmax = sync.txid_current
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [doc for doc in sync.sync(txmin=txmin, txmax=txmax)]
         assert docs == [
             {
                 "_id": "abc",
                 "_index": "testdb",
                 "_source": {
-                    "_meta": {},
+                    "_meta": {
+                        "book": {"isbn": ["abc"]},
+                    },
                     "description": "Tigers are fierce creatures",
                     "isbn": "abc",
                     "title": "The Tiger Club",
@@ -117,7 +116,9 @@ class TestRoot(object):
                 "_id": "def",
                 "_index": "testdb",
                 "_source": {
-                    "_meta": {},
+                    "_meta": {
+                        "book": {"isbn": ["def"]},
+                    },
                     "description": "Lion and the mouse",
                     "isbn": "def",
                     "title": "The Lion Club",
@@ -127,7 +128,7 @@ class TestRoot(object):
                 "_id": "ghi",
                 "_index": "testdb",
                 "_source": {
-                    "_meta": {},
+                    "_meta": {"book": {"isbn": ["ghi"]}},
                     "description": "Rabbits on the run",
                     "isbn": "ghi",
                     "title": "The Rabbit Club",
@@ -151,15 +152,16 @@ class TestRoot(object):
             },
         }
         sync.nodes = nodes
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [doc for doc in sync.sync()]
         assert docs == [
             {
                 "_id": "abc",
                 "_index": "testdb",
                 "_source": {
-                    "_meta": {},
+                    "_meta": {
+                        "book": {"isbn": ["abc"]},
+                    },
                     "description": "Tigers are fierce creatures",
                     "book_isbn": "abc",
                     "book_title": "The Tiger Club",
@@ -169,7 +171,9 @@ class TestRoot(object):
                 "_id": "def",
                 "_index": "testdb",
                 "_source": {
-                    "_meta": {},
+                    "_meta": {
+                        "book": {"isbn": ["def"]},
+                    },
                     "description": "Lion and the mouse",
                     "book_isbn": "def",
                     "book_title": "The Lion Club",
@@ -179,7 +183,7 @@ class TestRoot(object):
                 "_id": "ghi",
                 "_index": "testdb",
                 "_source": {
-                    "_meta": {},
+                    "_meta": {"book": {"isbn": ["ghi"]}},
                     "description": "Rabbits on the run",
                     "book_isbn": "ghi",
                     "book_title": "The Rabbit Club",
@@ -195,8 +199,7 @@ class TestRoot(object):
             "columns": ["isbn", "title", "description", "xmin"],
         }
         sync.nodes = nodes
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         assert sorted(docs[0]["_source"].keys()) == sorted(
             ["isbn", "title", "description", "xmin", "_meta"]
@@ -207,8 +210,7 @@ class TestRoot(object):
         """Test the doc includes xmin column."""
         nodes = {"table": "book", "columns": ["isbn", "xmin"]}
         sync.nodes = nodes
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         assert "xmin" in docs[0]["_source"]
         assert_resync_empty(sync, nodes)
@@ -217,8 +219,7 @@ class TestRoot(object):
         """Test we include all columns when no columns are specified."""
         nodes = {"table": "book", "columns": []}
         sync.nodes = nodes
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         assert sorted(docs[0]["_source"].keys()) == sorted(
             [
@@ -236,8 +237,7 @@ class TestRoot(object):
 
         nodes = {"table": "book"}
         sync.nodes = nodes
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         assert sorted(docs[0]["_source"].keys()) == sorted(
             [
@@ -269,8 +269,7 @@ class TestRoot(object):
         # TODO also repeat this test for composite primary key
         nodes = {"table": "book", "columns": ["title"]}
         sync.nodes = nodes
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         assert "abc" == docs[0]["_id"]
         assert "def" == docs[1]["_id"]
@@ -281,8 +280,7 @@ class TestRoot(object):
         """Test the private key is contained in the doc."""
         nodes = {"table": "book", "columns": ["isbn"]}
         sync.nodes = nodes
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         assert "_meta" in docs[0]["_source"]
         assert_resync_empty(sync, nodes)
@@ -291,8 +289,7 @@ class TestRoot(object):
         """Ensure the doc only selected columns and builtins."""
         nodes = {"table": "book", "columns": ["isbn", "xmin"]}
         sync.nodes = nodes
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         sources = {doc["_id"]: doc["_source"] for doc in docs}
 
@@ -310,8 +307,7 @@ class TestRoot(object):
             ],
         }
         sync.nodes = nodes
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         sources = {doc["_id"]: doc["_source"] for doc in docs}
 
@@ -327,8 +323,7 @@ class TestRoot(object):
         )
         nodes = {"table": "book", "columns": ["copyright", "publisher_id"]}
         sync.nodes = nodes
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
 
         docs = [sort_list(doc) for doc in sync.sync()]
         sources = {doc["_id"]: doc["_source"] for doc in docs}
@@ -349,26 +344,25 @@ class TestRoot(object):
             "columns": ["isbn", "description", "copyright"],
         }
         sync.nodes = nodes
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [doc for doc in sync.sync()]
         sources = {doc["_id"]: doc["_source"] for doc in docs}
 
         assert sources == {
             "abc": {
-                "_meta": {},
+                "_meta": {"book": {"isbn": ["abc"]}},
                 "copyright": None,
                 "description": "Tigers are fierce creatures",
                 "isbn": "abc",
             },
             "def": {
-                "_meta": {},
+                "_meta": {"book": {"isbn": ["def"]}},
                 "copyright": None,
                 "description": "Lion and the mouse",
                 "isbn": "def",
             },
             "ghi": {
-                "_meta": {},
+                "_meta": {"book": {"isbn": ["ghi"]}},
                 "copyright": None,
                 "description": "Rabbits on the run",
                 "isbn": "ghi",
@@ -380,14 +374,13 @@ class TestRoot(object):
         """Private keys should be included even if null."""
         nodes = {"table": "book", "columns": ["description"]}
         sync.nodes = nodes
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [doc for doc in sync.sync()]
         sources = {doc["_id"]: doc["_source"] for doc in docs}
 
-        assert sources["abc"]["_meta"] == {}
-        assert sources["def"]["_meta"] == {}
-        assert sources["ghi"]["_meta"] == {}
+        assert sources["abc"]["_meta"] == {"book": {"isbn": ["abc"]}}
+        assert sources["def"]["_meta"] == {"book": {"isbn": ["def"]}}
+        assert sources["ghi"]["_meta"] == {"book": {"isbn": ["ghi"]}}
         assert_resync_empty(sync, nodes)
 
     def test_node_include_table(self, sync, data):
@@ -414,21 +407,33 @@ class TestRoot(object):
         Note to self. I think this has been fixed. i.e we delete and then
         query ibsert
         """
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": {"table": "book", "columns": ["isbn", "title"]},
         }
-        sync = Sync(document)
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.search_client.refresh("testdb")
 
         docs = search(sync.search_client, "testdb")
 
         assert docs == [
-            {"_meta": {}, "isbn": "abc", "title": "The Tiger Club"},
-            {"_meta": {}, "isbn": "def", "title": "The Lion Club"},
-            {"_meta": {}, "isbn": "ghi", "title": "The Rabbit Club"},
+            {
+                "_meta": {"book": {"isbn": ["abc"]}},
+                "isbn": "abc",
+                "title": "The Tiger Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["def"]}},
+                "isbn": "def",
+                "title": "The Lion Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["ghi"]}},
+                "isbn": "ghi",
+                "title": "The Rabbit Club",
+            },
         ]
 
         session = sync.session
@@ -445,33 +450,64 @@ class TestRoot(object):
         docs = search(sync.search_client, "testdb")
 
         assert docs == [
-            {"_meta": {}, "isbn": "abc", "title": "The Tiger Club"},
-            {"_meta": {}, "isbn": "cba", "title": "The Tiger Club"},
-            {"_meta": {}, "isbn": "def", "title": "The Lion Club"},
-            {"_meta": {}, "isbn": "ghi", "title": "The Rabbit Club"},
+            {
+                "_meta": {"book": {"isbn": ["abc"]}},
+                "isbn": "abc",
+                "title": "The Tiger Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["cba"]}},
+                "isbn": "cba",
+                "title": "The Tiger Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["def"]}},
+                "isbn": "def",
+                "title": "The Lion Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["ghi"]}},
+                "isbn": "ghi",
+                "title": "The Rabbit Club",
+            },
         ]
-        assert_resync_empty(sync, document.get("node", {}))
+        assert_resync_empty(sync, doc.get("node", {}))
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     # TODO: Add another test like this and change
     # both primary key and non primary key column
     def test_update_primary_key_concurrent(self, data, book_cls):
         """Test sync updates primary_key and then sync in concurrent mode."""
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": {"table": "book", "columns": ["isbn", "title"]},
         }
-        sync = Sync(document)
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.search_client.refresh("testdb")
 
         docs = search(sync.search_client, "testdb")
 
         assert docs == [
-            {"_meta": {}, "isbn": "abc", "title": "The Tiger Club"},
-            {"_meta": {}, "isbn": "def", "title": "The Lion Club"},
-            {"_meta": {}, "isbn": "ghi", "title": "The Rabbit Club"},
+            {
+                "_meta": {"book": {"isbn": ["abc"]}},
+                "isbn": "abc",
+                "title": "The Tiger Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["def"]}},
+                "isbn": "def",
+                "title": "The Lion Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["ghi"]}},
+                "isbn": "ghi",
+                "title": "The Rabbit Club",
+            },
         ]
 
         session = sync.session
@@ -504,7 +540,7 @@ class TestRoot(object):
                             "pgsync.sync.Sync.status",
                             side_effect=noop,
                         ):
-                            sync.receive(NTHREADS_POLLDB)
+                            sync.receive()
                             sync.search_client.refresh("testdb")
 
         docs = search(sync.search_client, "testdb")
@@ -514,21 +550,36 @@ class TestRoot(object):
         assert "abc" not in [doc["isbn"] for doc in docs]
 
         assert docs == [
-            {"_meta": {}, "isbn": "cba", "title": "The Tiger Club"},
-            {"_meta": {}, "isbn": "def", "title": "The Lion Club"},
-            {"_meta": {}, "isbn": "ghi", "title": "The Rabbit Club"},
+            {
+                "_meta": {"book": {"isbn": ["cba"]}},
+                "isbn": "cba",
+                "title": "The Tiger Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["def"]}},
+                "isbn": "def",
+                "title": "The Lion Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["ghi"]}},
+                "isbn": "ghi",
+                "title": "The Rabbit Club",
+            },
         ]
-        assert_resync_empty(sync, document.get("node", {}))
+        assert_resync_empty(sync, doc.get("node", {}))
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_insert_non_concurrent(self, data, book_cls):
         """Test sync insert and then sync in non-concurrent mode."""
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": {"table": "book", "columns": ["isbn", "title"]},
         }
-        sync = Sync(document)
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.search_client.refresh("testdb")
 
@@ -537,9 +588,21 @@ class TestRoot(object):
         docs = search(sync.search_client, "testdb")
 
         assert docs == [
-            {"_meta": {}, "isbn": "abc", "title": "The Tiger Club"},
-            {"_meta": {}, "isbn": "def", "title": "The Lion Club"},
-            {"_meta": {}, "isbn": "ghi", "title": "The Rabbit Club"},
+            {
+                "_meta": {"book": {"isbn": ["abc"]}},
+                "isbn": "abc",
+                "title": "The Tiger Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["def"]}},
+                "isbn": "def",
+                "title": "The Lion Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["ghi"]}},
+                "isbn": "ghi",
+                "title": "The Rabbit Club",
+            },
         ]
         with subtransactions(session):
             session.execute(
@@ -554,22 +617,41 @@ class TestRoot(object):
         docs = search(sync.search_client, "testdb")
 
         assert docs == [
-            {"_meta": {}, "isbn": "abc", "title": "The Tiger Club"},
-            {"_meta": {}, "isbn": "def", "title": "The Lion Club"},
-            {"_meta": {}, "isbn": "ghi", "title": "The Rabbit Club"},
-            {"_meta": {}, "isbn": "xyz", "title": "Encyclopedia"},
+            {
+                "_meta": {"book": {"isbn": ["abc"]}},
+                "isbn": "abc",
+                "title": "The Tiger Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["def"]}},
+                "isbn": "def",
+                "title": "The Lion Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["ghi"]}},
+                "isbn": "ghi",
+                "title": "The Rabbit Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["xyz"]}},
+                "isbn": "xyz",
+                "title": "Encyclopedia",
+            },
         ]
-        assert_resync_empty(sync, document.get("node", {}))
+        assert_resync_empty(sync, doc.get("node", {}))
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_update_non_concurrent(self, data, book_cls):
         """Test sync update and then sync in non-concurrent mode."""
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": {"table": "book", "columns": ["isbn", "title"]},
         }
-        sync = Sync(document)
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.search_client.refresh("testdb")
 
@@ -578,9 +660,21 @@ class TestRoot(object):
         docs = search(sync.search_client, "testdb")
 
         assert docs == [
-            {"_meta": {}, "isbn": "abc", "title": "The Tiger Club"},
-            {"_meta": {}, "isbn": "def", "title": "The Lion Club"},
-            {"_meta": {}, "isbn": "ghi", "title": "The Rabbit Club"},
+            {
+                "_meta": {"book": {"isbn": ["abc"]}},
+                "isbn": "abc",
+                "title": "The Tiger Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["def"]}},
+                "isbn": "def",
+                "title": "The Lion Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["ghi"]}},
+                "isbn": "ghi",
+                "title": "The Rabbit Club",
+            },
         ]
 
         with subtransactions(session):
@@ -596,21 +690,36 @@ class TestRoot(object):
         docs = search(sync.search_client, "testdb")
 
         assert docs == [
-            {"_meta": {}, "isbn": "abc", "title": "Tiger Club"},
-            {"_meta": {}, "isbn": "def", "title": "The Lion Club"},
-            {"_meta": {}, "isbn": "ghi", "title": "The Rabbit Club"},
+            {
+                "_meta": {"book": {"isbn": ["abc"]}},
+                "isbn": "abc",
+                "title": "Tiger Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["def"]}},
+                "isbn": "def",
+                "title": "The Lion Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["ghi"]}},
+                "isbn": "ghi",
+                "title": "The Rabbit Club",
+            },
         ]
-        assert_resync_empty(sync, document.get("node", {}))
+        assert_resync_empty(sync, doc.get("node", {}))
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_update_concurrent(self, data, book_cls):
         """Test sync update and then sync in concurrent mode."""
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": {"table": "book", "columns": ["isbn", "title"]},
         }
-        sync = Sync(document)
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.search_client.refresh("testdb")
 
@@ -619,9 +728,21 @@ class TestRoot(object):
         docs = search(sync.search_client, "testdb")
 
         assert docs == [
-            {"_meta": {}, "isbn": "abc", "title": "The Tiger Club"},
-            {"_meta": {}, "isbn": "def", "title": "The Lion Club"},
-            {"_meta": {}, "isbn": "ghi", "title": "The Rabbit Club"},
+            {
+                "_meta": {"book": {"isbn": ["abc"]}},
+                "isbn": "abc",
+                "title": "The Tiger Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["def"]}},
+                "isbn": "def",
+                "title": "The Lion Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["ghi"]}},
+                "isbn": "ghi",
+                "title": "The Rabbit Club",
+            },
         ]
 
         def pull():
@@ -652,36 +773,63 @@ class TestRoot(object):
                             "pgsync.sync.Sync.status",
                             side_effect=noop,
                         ):
-                            sync.receive(NTHREADS_POLLDB)
+                            sync.receive()
                             sync.search_client.refresh("testdb")
 
         docs = search(sync.search_client, "testdb")
 
         assert docs == [
-            {"_meta": {}, "isbn": "abc", "title": "Tiger Club"},
-            {"_meta": {}, "isbn": "def", "title": "The Lion Club"},
-            {"_meta": {}, "isbn": "ghi", "title": "The Rabbit Club"},
+            {
+                "_meta": {"book": {"isbn": ["abc"]}},
+                "isbn": "abc",
+                "title": "Tiger Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["def"]}},
+                "isbn": "def",
+                "title": "The Lion Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["ghi"]}},
+                "isbn": "ghi",
+                "title": "The Rabbit Club",
+            },
         ]
-        assert_resync_empty(sync, document.get("node", {}))
+        assert_resync_empty(sync, doc.get("node", {}))
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_delete_concurrent(self, data, book_cls):
         """Test sync delete and then sync in concurrent mode."""
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": {"table": "book", "columns": ["isbn", "title"]},
         }
-        sync = Sync(document)
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.search_client.refresh("testdb")
         session = sync.session
         docs = search(sync.search_client, "testdb")
 
         assert docs == [
-            {"_meta": {}, "isbn": "abc", "title": "The Tiger Club"},
-            {"_meta": {}, "isbn": "def", "title": "The Lion Club"},
-            {"_meta": {}, "isbn": "ghi", "title": "The Rabbit Club"},
+            {
+                "_meta": {"book": {"isbn": ["abc"]}},
+                "isbn": "abc",
+                "title": "The Tiger Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["def"]}},
+                "isbn": "def",
+                "title": "The Lion Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["ghi"]}},
+                "isbn": "ghi",
+                "title": "The Rabbit Club",
+            },
         ]
 
         def pull():
@@ -712,17 +860,28 @@ class TestRoot(object):
                             "pgsync.sync.Sync.status",
                             side_effect=noop,
                         ):
-                            sync.receive(NTHREADS_POLLDB)
+                            sync.receive()
                             sync.search_client.refresh("testdb")
 
         docs = search(sync.search_client, "testdb")
 
         assert docs == [
-            {"_meta": {}, "isbn": "def", "title": "The Lion Club"},
-            {"_meta": {}, "isbn": "ghi", "title": "The Rabbit Club"},
+            {
+                "_meta": {"book": {"isbn": ["def"]}},
+                "isbn": "def",
+                "title": "The Lion Club",
+            },
+            {
+                "_meta": {"book": {"isbn": ["ghi"]}},
+                "isbn": "ghi",
+                "title": "The Rabbit Club",
+            },
         ]
-        assert_resync_empty(sync, document.get("node", {}))
+        assert_resync_empty(sync, doc.get("node", {}))
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_truncate(self, data, book_cls):
         """Test truncate."""
