@@ -4,7 +4,6 @@ import mock
 import psycopg2
 import pytest
 
-from pgsync import settings
 from pgsync.base import subtransactions
 from pgsync.exc import (
     ForeignKeyError,
@@ -14,21 +13,30 @@ from pgsync.exc import (
     RelationshipVariantError,
 )
 from pgsync.node import Tree
+from pgsync.settings import IS_MYSQL_COMPAT
 from pgsync.singleton import Singleton
 from pgsync.sync import Sync
 
 from .testing_utils import assert_resync_empty, noop, search, sort_list
 
 
+@pytest.mark.skipif(
+    IS_MYSQL_COMPAT,
+    reason="Skipped because IS_MYSQL_COMPAT env var is set",
+)
 @pytest.mark.usefixtures("table_creator")
 class TestParentSingleChildFkOnChild(object):
     """Root and single child node tests."""
 
     @pytest.fixture(scope="function")
     def data(self, sync, book_cls, rating_cls):
-
         session = sync.session
 
+        # Clean up any existing data first
+        with subtransactions(session):
+            session.execute(rating_cls.__table__.delete())
+            session.execute(book_cls.__table__.delete())
+
         books = [
             book_cls(
                 isbn="abc",
@@ -95,8 +103,7 @@ class TestParentSingleChildFkOnChild(object):
             raise
 
         sync.redis.delete()
-        session.connection().engine.connect().close()
-        session.connection().engine.dispose()
+        session.connection().engine.dispose(close=True)
         sync.search_client.close()
 
     def test_relationship_object_one_to_one(self, sync, data):
@@ -114,12 +121,11 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
-            "_meta": {"rating": {"id": [1]}},
+            "_meta": {"book": {"isbn": ["abc"]}, "rating": {"id": [1]}},
             "description": "Tigers are fierce creatures",
             "isbn": "abc",
             "rating": {"id": 1, "value": 1.1},
@@ -128,7 +134,7 @@ class TestParentSingleChildFkOnChild(object):
 
         assert docs[1]["_id"] == "def"
         assert docs[1]["_source"] == {
-            "_meta": {"rating": {"id": [2]}},
+            "_meta": {"book": {"isbn": ["def"]}, "rating": {"id": [2]}},
             "description": "Lion and the mouse",
             "isbn": "def",
             "rating": {"id": 2, "value": 2.2},
@@ -137,7 +143,7 @@ class TestParentSingleChildFkOnChild(object):
 
         assert docs[2]["_id"] == "ghi"
         assert docs[2]["_source"] == {
-            "_meta": {"rating": {"id": [3]}},
+            "_meta": {"book": {"isbn": ["ghi"]}, "rating": {"id": [3]}},
             "description": "Rabbits on the run",
             "isbn": "ghi",
             "rating": {"id": 3, "value": 3.3},
@@ -160,14 +166,13 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         docs = sorted(docs, key=lambda k: k["_id"])
 
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
-            "_meta": {"rating": {"id": [1]}},
+            "_meta": {"book": {"isbn": ["abc"]}, "rating": {"id": [1]}},
             "description": "Tigers are fierce creatures",
             "isbn": "abc",
             "rating": [{"id": 1, "value": 1.1}],
@@ -176,7 +181,7 @@ class TestParentSingleChildFkOnChild(object):
 
         assert docs[1]["_id"] == "def"
         assert docs[1]["_source"] == {
-            "_meta": {"rating": {"id": [2]}},
+            "_meta": {"book": {"isbn": ["def"]}, "rating": {"id": [2]}},
             "description": "Lion and the mouse",
             "isbn": "def",
             "rating": [{"id": 2, "value": 2.2}],
@@ -185,7 +190,7 @@ class TestParentSingleChildFkOnChild(object):
 
         assert docs[2]["_id"] == "ghi"
         assert docs[2]["_source"] == {
-            "_meta": {"rating": {"id": [3]}},
+            "_meta": {"book": {"isbn": ["ghi"]}, "rating": {"id": [3]}},
             "description": "Rabbits on the run",
             "isbn": "ghi",
             "rating": [{"id": 3, "value": 3.3}],
@@ -208,12 +213,11 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
-            "_meta": {"rating": {"id": [1]}},
+            "_meta": {"book": {"isbn": ["abc"]}, "rating": {"id": [1]}},
             "description": "Tigers are fierce creatures",
             "isbn": "abc",
             "rating": 1.1,
@@ -222,7 +226,7 @@ class TestParentSingleChildFkOnChild(object):
 
         assert docs[1]["_id"] == "def"
         assert docs[1]["_source"] == {
-            "_meta": {"rating": {"id": [2]}},
+            "_meta": {"book": {"isbn": ["def"]}, "rating": {"id": [2]}},
             "description": "Lion and the mouse",
             "isbn": "def",
             "rating": 2.2,
@@ -231,7 +235,7 @@ class TestParentSingleChildFkOnChild(object):
 
         assert docs[2]["_id"] == "ghi"
         assert docs[2]["_source"] == {
-            "_meta": {"rating": {"id": [3]}},
+            "_meta": {"book": {"isbn": ["ghi"]}, "rating": {"id": [3]}},
             "description": "Rabbits on the run",
             "isbn": "ghi",
             "rating": 3.3,
@@ -254,13 +258,12 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         docs = sorted(docs, key=lambda k: k["_id"])
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
-            "_meta": {"rating": {"id": [1]}},
+            "_meta": {"book": {"isbn": ["abc"]}, "rating": {"id": [1]}},
             "description": "Tigers are fierce creatures",
             "isbn": "abc",
             "rating": [1.1],
@@ -269,7 +272,7 @@ class TestParentSingleChildFkOnChild(object):
 
         assert docs[1]["_id"] == "def"
         assert docs[1]["_source"] == {
-            "_meta": {"rating": {"id": [2]}},
+            "_meta": {"book": {"isbn": ["def"]}, "rating": {"id": [2]}},
             "description": "Lion and the mouse",
             "isbn": "def",
             "rating": [2.2],
@@ -278,7 +281,7 @@ class TestParentSingleChildFkOnChild(object):
 
         assert docs[2]["_id"] == "ghi"
         assert docs[2]["_source"] == {
-            "_meta": {"rating": {"id": [3]}},
+            "_meta": {"book": {"isbn": ["ghi"]}, "rating": {"id": [3]}},
             "description": "Rabbits on the run",
             "isbn": "ghi",
             "rating": [3.3],
@@ -302,12 +305,11 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
-            "_meta": {"rating": {"id": [1]}},
+            "_meta": {"book": {"isbn": ["abc"]}, "rating": {"id": [1]}},
             "description": "Tigers are fierce creatures",
             "isbn": "abc",
             "rating_x": 1.1,
@@ -316,7 +318,7 @@ class TestParentSingleChildFkOnChild(object):
 
         assert docs[1]["_id"] == "def"
         assert docs[1]["_source"] == {
-            "_meta": {"rating": {"id": [2]}},
+            "_meta": {"book": {"isbn": ["def"]}, "rating": {"id": [2]}},
             "description": "Lion and the mouse",
             "isbn": "def",
             "rating_x": 2.2,
@@ -325,7 +327,7 @@ class TestParentSingleChildFkOnChild(object):
 
         assert docs[2]["_id"] == "ghi"
         assert docs[2]["_source"] == {
-            "_meta": {"rating": {"id": [3]}},
+            "_meta": {"book": {"isbn": ["ghi"]}, "rating": {"id": [3]}},
             "description": "Rabbits on the run",
             "isbn": "ghi",
             "rating_x": 3.3,
@@ -350,12 +352,11 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
-            "_meta": {"rating": {"id": [1]}},
+            "_meta": {"book": {"isbn": ["abc"]}, "rating": {"id": [1]}},
             "description": "Tigers are fierce creatures",
             "isbn": "abc",
             "rating": 1.1,
@@ -385,13 +386,12 @@ class TestParentSingleChildFkOnChild(object):
             ],
         }
         sync.nodes = nodes
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
 
         assert docs[0]["_source"] == {
-            "_meta": {"rating": {"id": [1]}},
+            "_meta": {"book": {"isbn": ["abc"]}, "rating": {"id": [1]}},
             "book_isbn": "abc",
             "book_title": "The Tiger Club",
             "description": "Tigers are fierce creatures",
@@ -399,7 +399,7 @@ class TestParentSingleChildFkOnChild(object):
         }
         assert docs[1]["_id"] == "def"
         assert docs[1]["_source"] == {
-            "_meta": {"rating": {"id": [2]}},
+            "_meta": {"book": {"isbn": ["def"]}, "rating": {"id": [2]}},
             "book_isbn": "def",
             "book_title": "The Lion Club",
             "description": "Lion and the mouse",
@@ -408,7 +408,7 @@ class TestParentSingleChildFkOnChild(object):
 
         assert docs[2]["_id"] == "ghi"
         assert docs[2]["_source"] == {
-            "_meta": {"rating": {"id": [3]}},
+            "_meta": {"book": {"isbn": ["ghi"]}, "rating": {"id": [3]}},
             "book_isbn": "ghi",
             "book_title": "The Rabbit Club",
             "description": "Rabbits on the run",
@@ -431,8 +431,7 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
 
         fields = ["_meta", "description", "isbn", "rating", "title"]
@@ -459,11 +458,10 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[2]["_source"] == {
-            "_meta": {"rating": {"id": [3]}},
+            "_meta": {"book": {"isbn": ["ghi"]}, "rating": {"id": [3]}},
             "copyright": None,
             "description": "Rabbits on the run",
             "isbn": "ghi",
@@ -488,7 +486,7 @@ class TestParentSingleChildFkOnChild(object):
         }
         sync.search_client.close()
         with pytest.raises(RelationshipTypeError) as excinfo:
-            Tree(sync.models).build(nodes)
+            Tree(sync.models, nodes, database=sync.database)
         assert 'Relationship type "qwerty" is invalid' in str(excinfo.value)
 
     def test_invalid_relationship_variant(self, sync):
@@ -506,7 +504,7 @@ class TestParentSingleChildFkOnChild(object):
         }
         sync.search_client.close()
         with pytest.raises(RelationshipVariantError) as excinfo:
-            Tree(sync.models).build(nodes)
+            Tree(sync.models, nodes, database=sync.database)
         assert 'Relationship variant "abcdefg" is invalid' in str(
             excinfo.value
         )
@@ -523,7 +521,7 @@ class TestParentSingleChildFkOnChild(object):
         }
         sync.search_client.close()
         with pytest.raises(RelationshipAttributeError) as excinfo:
-            Tree(sync.models).build(nodes)
+            Tree(sync.models, nodes, database=sync.database)
         assert f"Relationship attribute {set(['foo'])} is invalid" in str(
             excinfo.value
         )
@@ -542,13 +540,21 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         sources = {doc["_id"]: doc["_source"] for doc in docs}
-        assert sources["abc"]["_meta"] == {"rating": {"id": [1]}}
-        assert sources["def"]["_meta"] == {"rating": {"id": [2]}}
-        assert sources["ghi"]["_meta"] == {"rating": {"id": [3]}}
+        assert sources["abc"]["_meta"] == {
+            "book": {"isbn": ["abc"]},
+            "rating": {"id": [1]},
+        }
+        assert sources["def"]["_meta"] == {
+            "book": {"isbn": ["def"]},
+            "rating": {"id": [2]},
+        }
+        assert sources["ghi"]["_meta"] == {
+            "book": {"isbn": ["ghi"]},
+            "rating": {"id": [3]},
+        }
         assert_resync_empty(sync, nodes)
 
     def test_missing_foreign_keys(self, sync, data):
@@ -565,8 +571,7 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         with pytest.raises(ForeignKeyError) as excinfo:
             [sort_list(doc) for doc in sync.sync()]
         msg = (
@@ -589,7 +594,7 @@ class TestParentSingleChildFkOnChild(object):
         self, data, book_cls, rating_cls, engine
     ):
         """Test sync updates primary_key then sync in non-concurrent mode."""
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": {
@@ -607,7 +612,7 @@ class TestParentSingleChildFkOnChild(object):
                 ],
             },
         }
-        sync = Sync(document)
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.search_client.refresh("testdb")
 
@@ -615,19 +620,19 @@ class TestParentSingleChildFkOnChild(object):
 
         assert docs == [
             {
-                "_meta": {"rating": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "rating": {"id": [1]}},
                 "isbn": "abc",
                 "rating": {"id": 1, "value": 1.1},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"rating": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "rating": {"id": [2]}},
                 "isbn": "def",
                 "rating": {"id": 2, "value": 2.2},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"rating": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["ghi"]}, "rating": {"id": [3]}},
                 "isbn": "ghi",
                 "rating": {"id": 3, "value": 3.3},
                 "title": "The Rabbit Club",
@@ -657,38 +662,41 @@ class TestParentSingleChildFkOnChild(object):
         docs = search(sync.search_client, "testdb")
         assert docs == [
             {
-                "_meta": {"rating": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "rating": {"id": [1]}},
                 "isbn": "abc",
                 "rating": {"id": 1, "value": 1.1},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"rating": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "rating": {"id": [2]}},
                 "isbn": "def",
                 "rating": {"id": 2, "value": 2.2},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {},
+                "_meta": {"book": {"isbn": ["ghi"]}},
                 "isbn": "ghi",
                 "rating": None,
                 "title": "The Rabbit Club",
             },
             {
-                "_meta": {"rating": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["xyz"]}, "rating": {"id": [3]}},
                 "isbn": "xyz",
                 "rating": {"id": 3, "value": 3.3},
                 "title": "Milli and the Ants",
             },
         ]
-        assert_resync_empty(sync, document.get("node", {}))
+        assert_resync_empty(sync, doc.get("node", {}))
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     # TODO: Add another test like this and change
     # both primary key and non pkey column
     def test_update_primary_key_concurrent(self, data, book_cls, rating_cls):
         """Test sync updates primary_key and then sync in concurrent mode."""
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": {
@@ -706,26 +714,26 @@ class TestParentSingleChildFkOnChild(object):
                 ],
             },
         }
-        sync = Sync(document)
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.search_client.refresh("testdb")
         docs = search(sync.search_client, "testdb")
 
         assert docs == [
             {
-                "_meta": {"rating": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "rating": {"id": [1]}},
                 "isbn": "abc",
                 "rating": {"id": 1, "value": 1.1},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"rating": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "rating": {"id": [2]}},
                 "isbn": "def",
                 "rating": {"id": 2, "value": 2.2},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"rating": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["ghi"]}, "rating": {"id": [3]}},
                 "isbn": "ghi",
                 "rating": {"id": 3, "value": 3.3},
                 "title": "The Rabbit Club",
@@ -771,42 +779,45 @@ class TestParentSingleChildFkOnChild(object):
                             "pgsync.sync.Sync.status",
                             side_effect=noop,
                         ):
-                            sync.receive(settings.NTHREADS_POLLDB)
+                            sync.receive()
                             sync.search_client.refresh("testdb")
 
         docs = search(sync.search_client, "testdb")
         assert docs == [
             {
-                "_meta": {"rating": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "rating": {"id": [1]}},
                 "isbn": "abc",
                 "rating": {"id": 1, "value": 1.1},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"rating": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "rating": {"id": [2]}},
                 "isbn": "def",
                 "rating": {"id": 2, "value": 2.2},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {},
+                "_meta": {"book": {"isbn": ["ghi"]}},
                 "isbn": "ghi",
                 "rating": None,
                 "title": "The Rabbit Club",
             },
             {
-                "_meta": {"rating": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["xyz"]}, "rating": {"id": [3]}},
                 "isbn": "xyz",
                 "rating": {"id": 3, "value": 3.3},
                 "title": "Milli and the Ants",
             },
         ]
-        assert_resync_empty(sync, document.get("node", {}))
+        assert_resync_empty(sync, doc.get("node", {}))
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_insert_non_concurrent(self, data, book_cls, rating_cls):
         """Test sync insert and then sync in non-concurrent mode."""
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": {
@@ -824,7 +835,7 @@ class TestParentSingleChildFkOnChild(object):
                 ],
             },
         }
-        sync = Sync(document)
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.search_client.refresh("testdb")
 
@@ -833,19 +844,19 @@ class TestParentSingleChildFkOnChild(object):
         docs = search(sync.search_client, "testdb")
         assert docs == [
             {
-                "_meta": {"rating": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "rating": {"id": [1]}},
                 "isbn": "abc",
                 "rating": {"id": 1, "value": 1.1},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"rating": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "rating": {"id": [2]}},
                 "isbn": "def",
                 "rating": {"id": 2, "value": 2.2},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"rating": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["ghi"]}, "rating": {"id": [3]}},
                 "isbn": "ghi",
                 "rating": {"id": 3, "value": 3.3},
                 "title": "The Rabbit Club",
@@ -878,38 +889,41 @@ class TestParentSingleChildFkOnChild(object):
 
         assert docs == [
             {
-                "_meta": {"rating": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "rating": {"id": [1]}},
                 "isbn": "abc",
                 "rating": {"id": 1, "value": 1.1},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"rating": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "rating": {"id": [2]}},
                 "isbn": "def",
                 "rating": {"id": 2, "value": 2.2},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"rating": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["ghi"]}, "rating": {"id": [3]}},
                 "isbn": "ghi",
                 "rating": {"id": 3, "value": 3.3},
                 "title": "The Rabbit Club",
             },
             {
-                "_meta": {"rating": {"id": [99]}},
+                "_meta": {"book": {"isbn": ["xyz"]}, "rating": {"id": [99]}},
                 "isbn": "xyz",
                 "rating": {"id": 99, "value": 4.4},
                 "title": "Encyclopedia",
             },
         ]
-        assert_resync_empty(sync, document.get("node", {}))
+        assert_resync_empty(sync, doc.get("node", {}))
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_update_non_primary_key_non_concurrent(
         self, data, book_cls, rating_cls
     ):
         """Test sync update and then sync in non-concurrent mode."""
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": {
@@ -927,7 +941,7 @@ class TestParentSingleChildFkOnChild(object):
                 ],
             },
         }
-        sync = Sync(document)
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.search_client.refresh("testdb")
 
@@ -935,19 +949,19 @@ class TestParentSingleChildFkOnChild(object):
 
         assert docs == [
             {
-                "_meta": {"rating": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "rating": {"id": [1]}},
                 "isbn": "abc",
                 "rating": {"id": 1, "value": 1.1},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"rating": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "rating": {"id": [2]}},
                 "isbn": "def",
                 "rating": {"id": 2, "value": 2.2},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"rating": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["ghi"]}, "rating": {"id": [3]}},
                 "isbn": "ghi",
                 "rating": {"id": 3, "value": 3.3},
                 "title": "The Rabbit Club",
@@ -974,32 +988,35 @@ class TestParentSingleChildFkOnChild(object):
 
         assert docs == [
             {
-                "_meta": {"rating": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "rating": {"id": [1]}},
                 "isbn": "abc",
                 "rating": {"id": 1, "value": 1.1},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"rating": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "rating": {"id": [2]}},
                 "isbn": "def",
                 "rating": {"id": 2, "value": 2.2},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"rating": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["ghi"]}, "rating": {"id": [3]}},
                 "isbn": "ghi",
                 "rating": {"id": 3, "value": 4.4},
                 "title": "The Rabbit Club",
             },
         ]
-        assert_resync_empty(sync, document.get("node", {}))
+        assert_resync_empty(sync, doc.get("node", {}))
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_update_non_primary_key_concurrent(
         self, data, book_cls, rating_cls
     ):
         """Test sync update and then sync in concurrent mode."""
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": {
@@ -1017,7 +1034,7 @@ class TestParentSingleChildFkOnChild(object):
                 ],
             },
         }
-        sync = Sync(document)
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.search_client.refresh("testdb")
 
@@ -1025,19 +1042,19 @@ class TestParentSingleChildFkOnChild(object):
 
         assert docs == [
             {
-                "_meta": {"rating": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "rating": {"id": [1]}},
                 "isbn": "abc",
                 "rating": {"id": 1, "value": 1.1},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"rating": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "rating": {"id": [2]}},
                 "isbn": "def",
                 "rating": {"id": 2, "value": 2.2},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"rating": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["ghi"]}, "rating": {"id": [3]}},
                 "isbn": "ghi",
                 "rating": {"id": 3, "value": 3.3},
                 "title": "The Rabbit Club",
@@ -1074,37 +1091,40 @@ class TestParentSingleChildFkOnChild(object):
                             "pgsync.sync.Sync.status",
                             side_effect=noop,
                         ):
-                            sync.receive(settings.NTHREADS_POLLDB)
+                            sync.receive()
                             sync.search_client.refresh("testdb")
 
         docs = search(sync.search_client, "testdb")
 
         assert docs == [
             {
-                "_meta": {"rating": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "rating": {"id": [1]}},
                 "isbn": "abc",
                 "rating": {"id": 1, "value": 1.1},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"rating": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "rating": {"id": [2]}},
                 "isbn": "def",
                 "rating": {"id": 2, "value": 2.2},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"rating": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["ghi"]}, "rating": {"id": [3]}},
                 "isbn": "ghi",
                 "rating": {"id": 3, "value": 4.4},
                 "title": "The Rabbit Club",
             },
         ]
-        assert_resync_empty(sync, document.get("node", {}))
+        assert_resync_empty(sync, doc.get("node", {}))
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_delete_concurrent(self, data, book_cls, rating_cls):
         """Test sync delete and then sync in concurrent mode."""
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": {
@@ -1123,7 +1143,7 @@ class TestParentSingleChildFkOnChild(object):
             },
         }
 
-        sync = Sync(document)
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.search_client.refresh("testdb")
 
@@ -1131,19 +1151,19 @@ class TestParentSingleChildFkOnChild(object):
 
         assert docs == [
             {
-                "_meta": {"rating": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "rating": {"id": [1]}},
                 "isbn": "abc",
                 "rating": {"id": 1, "value": 1.1},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"rating": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "rating": {"id": [2]}},
                 "isbn": "def",
                 "rating": {"id": 2, "value": 2.2},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"rating": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["ghi"]}, "rating": {"id": [3]}},
                 "isbn": "ghi",
                 "rating": {"id": 3, "value": 3.3},
                 "title": "The Rabbit Club",
@@ -1194,29 +1214,32 @@ class TestParentSingleChildFkOnChild(object):
                             "pgsync.sync.Sync.status",
                             side_effect=noop,
                         ):
-                            sync.receive(settings.NTHREADS_POLLDB)
+                            sync.receive()
                             sync.search_client.refresh("testdb")
 
         docs = search(sync.search_client, "testdb")
         assert docs == [
             {
-                "_meta": {"rating": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "rating": {"id": [1]}},
                 "isbn": "abc",
                 "rating": {"id": 1, "value": 1.1},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"rating": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "rating": {"id": [2]}},
                 "isbn": "def",
                 "rating": {"id": 2, "value": 2.2},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"rating": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["xyz"]}, "rating": {"id": [3]}},
                 "isbn": "xyz",
                 "rating": {"id": 3, "value": 3.3},
                 "title": "The End of time",
             },
         ]
-        assert_resync_empty(sync, document.get("node", {}))
+        assert_resync_empty(sync, doc.get("node", {}))
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
