@@ -13,22 +13,30 @@ from pgsync.exc import (
     RelationshipVariantError,
 )
 from pgsync.node import Tree
-from pgsync.settings import NTHREADS_POLLDB
+from pgsync.settings import IS_MYSQL_COMPAT
 from pgsync.singleton import Singleton
 from pgsync.sync import Sync
 
 from .testing_utils import assert_resync_empty, noop, search, sort_list
 
 
+@pytest.mark.skipif(
+    IS_MYSQL_COMPAT,
+    reason="Skipped because IS_MYSQL_COMPAT env var is set",
+)
 @pytest.mark.usefixtures("table_creator")
 class TestParentSingleChildFkOnParent(object):
     """Root and single child node tests."""
 
     @pytest.fixture(scope="function")
     def data(self, sync, book_cls, publisher_cls):
-
         session = sync.session
 
+        # Clean up any existing data first
+        with subtransactions(session):
+            session.execute(book_cls.__table__.delete())
+            session.execute(publisher_cls.__table__.delete())
+
         books = [
             book_cls(
                 isbn="abc",
@@ -86,8 +94,7 @@ class TestParentSingleChildFkOnParent(object):
             raise
 
         sync.redis.delete()
-        session.connection().engine.connect().close()
-        session.connection().engine.dispose()
+        session.connection().engine.dispose(close=True)
         sync.search_client.close()
 
     def test_relationship_object_one_to_one(self, sync, data):
@@ -105,12 +112,11 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
-            "_meta": {"publisher": {"id": [1]}},
+            "_meta": {"book": {"isbn": ["abc"]}, "publisher": {"id": [1]}},
             "description": "Tigers are fierce creatures",
             "isbn": "abc",
             "publisher": {"id": 1, "name": "Tiger publishing"},
@@ -119,7 +125,7 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs[1]["_id"] == "def"
         assert docs[1]["_source"] == {
-            "_meta": {"publisher": {"id": [2]}},
+            "_meta": {"book": {"isbn": ["def"]}, "publisher": {"id": [2]}},
             "description": "Lion and the mouse",
             "isbn": "def",
             "publisher": {"id": 2, "name": "Lion publishing"},
@@ -128,7 +134,7 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs[2]["_id"] == "ghi"
         assert docs[2]["_source"] == {
-            "_meta": {"publisher": {"id": [3]}},
+            "_meta": {"book": {"isbn": ["ghi"]}, "publisher": {"id": [3]}},
             "description": "Rabbits on the run",
             "isbn": "ghi",
             "publisher": {"id": 3, "name": "Hop Bunny publishing"},
@@ -151,14 +157,13 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         docs = sorted(docs, key=lambda k: k["_id"])
 
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
-            "_meta": {"publisher": {"id": [1]}},
+            "_meta": {"book": {"isbn": ["abc"]}, "publisher": {"id": [1]}},
             "description": "Tigers are fierce creatures",
             "isbn": "abc",
             "publisher": [{"id": 1, "name": "Tiger publishing"}],
@@ -167,7 +172,7 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs[1]["_id"] == "def"
         assert docs[1]["_source"] == {
-            "_meta": {"publisher": {"id": [2]}},
+            "_meta": {"book": {"isbn": ["def"]}, "publisher": {"id": [2]}},
             "description": "Lion and the mouse",
             "isbn": "def",
             "publisher": [{"id": 2, "name": "Lion publishing"}],
@@ -176,7 +181,7 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs[2]["_id"] == "ghi"
         assert docs[2]["_source"] == {
-            "_meta": {"publisher": {"id": [3]}},
+            "_meta": {"book": {"isbn": ["ghi"]}, "publisher": {"id": [3]}},
             "description": "Rabbits on the run",
             "isbn": "ghi",
             "publisher": [{"id": 3, "name": "Hop Bunny publishing"}],
@@ -199,12 +204,11 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
-            "_meta": {"publisher": {"id": [1]}},
+            "_meta": {"book": {"isbn": ["abc"]}, "publisher": {"id": [1]}},
             "description": "Tigers are fierce creatures",
             "isbn": "abc",
             "publisher": "Tiger publishing",
@@ -213,7 +217,7 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs[1]["_id"] == "def"
         assert docs[1]["_source"] == {
-            "_meta": {"publisher": {"id": [2]}},
+            "_meta": {"book": {"isbn": ["def"]}, "publisher": {"id": [2]}},
             "description": "Lion and the mouse",
             "isbn": "def",
             "publisher": "Lion publishing",
@@ -222,7 +226,7 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs[2]["_id"] == "ghi"
         assert docs[2]["_source"] == {
-            "_meta": {"publisher": {"id": [3]}},
+            "_meta": {"book": {"isbn": ["ghi"]}, "publisher": {"id": [3]}},
             "description": "Rabbits on the run",
             "isbn": "ghi",
             "publisher": "Hop Bunny publishing",
@@ -245,13 +249,12 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         docs = sorted(docs, key=lambda k: k["_id"])
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
-            "_meta": {"publisher": {"id": [1]}},
+            "_meta": {"book": {"isbn": ["abc"]}, "publisher": {"id": [1]}},
             "description": "Tigers are fierce creatures",
             "isbn": "abc",
             "publisher": ["Tiger publishing"],
@@ -260,7 +263,7 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs[1]["_id"] == "def"
         assert docs[1]["_source"] == {
-            "_meta": {"publisher": {"id": [2]}},
+            "_meta": {"book": {"isbn": ["def"]}, "publisher": {"id": [2]}},
             "description": "Lion and the mouse",
             "isbn": "def",
             "publisher": ["Lion publishing"],
@@ -269,7 +272,7 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs[2]["_id"] == "ghi"
         assert docs[2]["_source"] == {
-            "_meta": {"publisher": {"id": [3]}},
+            "_meta": {"book": {"isbn": ["ghi"]}, "publisher": {"id": [3]}},
             "description": "Rabbits on the run",
             "isbn": "ghi",
             "publisher": ["Hop Bunny publishing"],
@@ -293,12 +296,11 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
-            "_meta": {"publisher": {"id": [1]}},
+            "_meta": {"book": {"isbn": ["abc"]}, "publisher": {"id": [1]}},
             "description": "Tigers are fierce creatures",
             "isbn": "abc",
             "publisher_x": "Tiger publishing",
@@ -307,7 +309,7 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs[1]["_id"] == "def"
         assert docs[1]["_source"] == {
-            "_meta": {"publisher": {"id": [2]}},
+            "_meta": {"book": {"isbn": ["def"]}, "publisher": {"id": [2]}},
             "description": "Lion and the mouse",
             "isbn": "def",
             "publisher_x": "Lion publishing",
@@ -316,7 +318,7 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs[2]["_id"] == "ghi"
         assert docs[2]["_source"] == {
-            "_meta": {"publisher": {"id": [3]}},
+            "_meta": {"book": {"isbn": ["ghi"]}, "publisher": {"id": [3]}},
             "description": "Rabbits on the run",
             "isbn": "ghi",
             "publisher_x": "Hop Bunny publishing",
@@ -341,12 +343,11 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
-            "_meta": {"publisher": {"id": [1]}},
+            "_meta": {"book": {"isbn": ["abc"]}, "publisher": {"id": [1]}},
             "description": "Tigers are fierce creatures",
             "isbn": "abc",
             "publisher": "Tiger publishing",
@@ -379,12 +380,11 @@ class TestParentSingleChildFkOnParent(object):
             ],
         }
         sync.nodes = nodes
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
-            "_meta": {"publisher": {"id": [1]}},
+            "_meta": {"book": {"isbn": ["abc"]}, "publisher": {"id": [1]}},
             "book_isbn": "abc",
             "book_title": "The Tiger Club",
             "description": "Tigers are fierce creatures",
@@ -396,7 +396,7 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs[1]["_id"] == "def"
         assert docs[1]["_source"] == {
-            "_meta": {"publisher": {"id": [2]}},
+            "_meta": {"book": {"isbn": ["def"]}, "publisher": {"id": [2]}},
             "book_isbn": "def",
             "book_title": "The Lion Club",
             "description": "Lion and the mouse",
@@ -408,7 +408,7 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs[2]["_id"] == "ghi"
         assert docs[2]["_source"] == {
-            "_meta": {"publisher": {"id": [3]}},
+            "_meta": {"book": {"isbn": ["ghi"]}, "publisher": {"id": [3]}},
             "book_isbn": "ghi",
             "book_title": "The Rabbit Club",
             "description": "Rabbits on the run",
@@ -435,8 +435,7 @@ class TestParentSingleChildFkOnParent(object):
             ],
         }
 
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         fields = ["_meta", "description", "isbn", "publisher", "title"]
         assert sorted(docs[0]["_source"].keys()) == sorted(fields)
@@ -462,11 +461,10 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[2]["_source"] == {
-            "_meta": {"publisher": {"id": [3]}},
+            "_meta": {"book": {"isbn": ["ghi"]}, "publisher": {"id": [3]}},
             "copyright": None,
             "description": "Rabbits on the run",
             "isbn": "ghi",
@@ -492,7 +490,7 @@ class TestParentSingleChildFkOnParent(object):
         sync.search_client.close()
         sync.tree.__post_init__()
         with pytest.raises(RelationshipTypeError) as excinfo:
-            Tree(sync.models).build(nodes)
+            Tree(sync.models, nodes, database=sync.database)
         assert 'Relationship type "qwerty" is invalid' in str(excinfo.value)
 
     def test_invalid_relationship_variant(self, sync):
@@ -511,7 +509,7 @@ class TestParentSingleChildFkOnParent(object):
         sync.search_client.close()
         sync.tree.__post_init__()
         with pytest.raises(RelationshipVariantError) as excinfo:
-            Tree(sync.models).build(nodes)
+            Tree(sync.models, nodes, database=sync.database)
         assert 'Relationship variant "abcdefg" is invalid' in str(
             excinfo.value
         )
@@ -529,7 +527,7 @@ class TestParentSingleChildFkOnParent(object):
         sync.search_client.close()
         sync.tree.__post_init__()
         with pytest.raises(RelationshipAttributeError) as excinfo:
-            Tree(sync.models).build(nodes)
+            Tree(sync.models, nodes, database=sync.database)
         assert f"Relationship attribute {set(['foo'])} is invalid" in str(
             excinfo.value
         )
@@ -548,13 +546,21 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         sources = {doc["_id"]: doc["_source"] for doc in docs}
-        assert sources["abc"]["_meta"] == {"publisher": {"id": [1]}}
-        assert sources["def"]["_meta"] == {"publisher": {"id": [2]}}
-        assert sources["ghi"]["_meta"] == {"publisher": {"id": [3]}}
+        assert sources["abc"]["_meta"] == {
+            "book": {"isbn": ["abc"]},
+            "publisher": {"id": [1]},
+        }
+        assert sources["def"]["_meta"] == {
+            "book": {"isbn": ["def"]},
+            "publisher": {"id": [2]},
+        }
+        assert sources["ghi"]["_meta"] == {
+            "book": {"isbn": ["ghi"]},
+            "publisher": {"id": [3]},
+        }
         assert_resync_empty(sync, nodes)
 
     def test_missing_foreign_keys(self, sync, data):
@@ -571,8 +577,7 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
-        sync.tree.__post_init__()
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         with pytest.raises(ForeignKeyError) as excinfo:
             [sort_list(doc) for doc in sync.sync()]
         msg = (
@@ -595,7 +600,7 @@ class TestParentSingleChildFkOnParent(object):
         self, data, book_cls, publisher_cls, engine
     ):
         """Test sync updates primary_key then sync in non-concurrent mode."""
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": {
@@ -613,7 +618,7 @@ class TestParentSingleChildFkOnParent(object):
                 ],
             },
         }
-        sync = Sync(document)
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.search_client.refresh("testdb")
 
@@ -621,19 +626,19 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs == [
             {
-                "_meta": {"publisher": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "publisher": {"id": [1]}},
                 "isbn": "abc",
                 "publisher": {"id": 1, "name": "Tiger publishing"},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"publisher": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "publisher": {"id": [2]}},
                 "isbn": "def",
                 "publisher": {"id": 2, "name": "Lion publishing"},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"publisher": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["ghi"]}, "publisher": {"id": [3]}},
                 "isbn": "ghi",
                 "publisher": {"id": 3, "name": "Hop Bunny publishing"},
                 "title": "The Rabbit Club",
@@ -664,26 +669,32 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs == [
             {
-                "_meta": {"publisher": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "publisher": {"id": [1]}},
                 "isbn": "abc",
                 "publisher": {"id": 1, "name": "Tiger publishing"},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"publisher": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "publisher": {"id": [2]}},
                 "isbn": "def",
                 "publisher": {"id": 2, "name": "Lion publishing"},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"publisher": {"id": [99]}},
+                "_meta": {
+                    "book": {"isbn": ["ghi"]},
+                    "publisher": {"id": [99]},
+                },
                 "isbn": "ghi",
                 "publisher": {"id": 99, "name": "Rabbit publishers"},
                 "title": "The Rabbit Club",
             },
         ]
-        assert_resync_empty(sync, document.get("node", {}))
+        assert_resync_empty(sync, doc.get("node", {}))
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     # TODO: Add another test like this and change
     # both primary key and non pkey column
@@ -691,7 +702,7 @@ class TestParentSingleChildFkOnParent(object):
         self, data, book_cls, publisher_cls
     ):
         """Test sync updates primary_key and then sync in concurrent mode."""
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": {
@@ -709,7 +720,7 @@ class TestParentSingleChildFkOnParent(object):
                 ],
             },
         }
-        sync = Sync(document)
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.search_client.refresh("testdb")
 
@@ -717,19 +728,19 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs == [
             {
-                "_meta": {"publisher": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "publisher": {"id": [1]}},
                 "isbn": "abc",
                 "publisher": {"id": 1, "name": "Tiger publishing"},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"publisher": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "publisher": {"id": [2]}},
                 "isbn": "def",
                 "publisher": {"id": 2, "name": "Lion publishing"},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"publisher": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["ghi"]}, "publisher": {"id": [3]}},
                 "isbn": "ghi",
                 "publisher": {"id": 3, "name": "Hop Bunny publishing"},
                 "title": "The Rabbit Club",
@@ -774,37 +785,43 @@ class TestParentSingleChildFkOnParent(object):
                             "pgsync.sync.Sync.status",
                             side_effect=noop,
                         ):
-                            sync.receive(NTHREADS_POLLDB)
+                            sync.receive()
                             sync.search_client.refresh("testdb")
 
         docs = search(sync.search_client, "testdb")
 
         assert docs == [
             {
-                "_meta": {"publisher": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "publisher": {"id": [1]}},
                 "isbn": "abc",
                 "publisher": {"id": 1, "name": "Tiger publishing"},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"publisher": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "publisher": {"id": [2]}},
                 "isbn": "def",
                 "publisher": {"id": 2, "name": "Lion publishing"},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"publisher": {"id": [99]}},
+                "_meta": {
+                    "book": {"isbn": ["ghi"]},
+                    "publisher": {"id": [99]},
+                },
                 "isbn": "ghi",
                 "publisher": {"id": 99, "name": "Rabbit publishers"},
                 "title": "The Rabbit Club",
             },
         ]
-        assert_resync_empty(sync, document.get("node", {}))
+        assert_resync_empty(sync, doc.get("node", {}))
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_insert_non_concurrent(self, data, book_cls, publisher_cls):
         """Test sync insert and then sync in non-concurrent mode."""
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": {
@@ -822,7 +839,7 @@ class TestParentSingleChildFkOnParent(object):
                 ],
             },
         }
-        sync = Sync(document)
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.search_client.refresh("testdb")
 
@@ -831,19 +848,19 @@ class TestParentSingleChildFkOnParent(object):
         docs = search(sync.search_client, "testdb")
         assert docs == [
             {
-                "_meta": {"publisher": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "publisher": {"id": [1]}},
                 "isbn": "abc",
                 "publisher": {"id": 1, "name": "Tiger publishing"},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"publisher": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "publisher": {"id": [2]}},
                 "isbn": "def",
                 "publisher": {"id": 2, "name": "Lion publishing"},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"publisher": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["ghi"]}, "publisher": {"id": [3]}},
                 "isbn": "ghi",
                 "publisher": {"id": 3, "name": "Hop Bunny publishing"},
                 "title": "The Rabbit Club",
@@ -873,38 +890,44 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs == [
             {
-                "_meta": {"publisher": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "publisher": {"id": [1]}},
                 "isbn": "abc",
                 "publisher": {"id": 1, "name": "Tiger publishing"},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"publisher": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "publisher": {"id": [2]}},
                 "isbn": "def",
                 "publisher": {"id": 2, "name": "Lion publishing"},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"publisher": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["ghi"]}, "publisher": {"id": [3]}},
                 "isbn": "ghi",
                 "publisher": {"id": 3, "name": "Hop Bunny publishing"},
                 "title": "The Rabbit Club",
             },
             {
-                "_meta": {"publisher": {"id": [99]}},
+                "_meta": {
+                    "book": {"isbn": ["xyz"]},
+                    "publisher": {"id": [99]},
+                },
                 "isbn": "xyz",
                 "publisher": {"id": 99, "name": "Rabbit publishers"},
                 "title": "Encyclopedia",
             },
         ]
-        assert_resync_empty(sync, document.get("node", {}))
+        assert_resync_empty(sync, doc.get("node", {}))
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_update_non_primary_key_non_concurrent(
         self, data, book_cls, publisher_cls
     ):
         """Test sync update and then sync in non-concurrent mode."""
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": {
@@ -922,7 +945,7 @@ class TestParentSingleChildFkOnParent(object):
                 ],
             },
         }
-        sync = Sync(document)
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.search_client.refresh("testdb")
 
@@ -930,19 +953,19 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs == [
             {
-                "_meta": {"publisher": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "publisher": {"id": [1]}},
                 "isbn": "abc",
                 "publisher": {"id": 1, "name": "Tiger publishing"},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"publisher": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "publisher": {"id": [2]}},
                 "isbn": "def",
                 "publisher": {"id": 2, "name": "Lion publishing"},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"publisher": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["ghi"]}, "publisher": {"id": [3]}},
                 "isbn": "ghi",
                 "publisher": {"id": 3, "name": "Hop Bunny publishing"},
                 "title": "The Rabbit Club",
@@ -969,32 +992,35 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs == [
             {
-                "_meta": {"publisher": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "publisher": {"id": [1]}},
                 "isbn": "abc",
                 "publisher": {"id": 1, "name": "Tiger publishing"},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"publisher": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "publisher": {"id": [2]}},
                 "isbn": "def",
                 "publisher": {"id": 2, "name": "Lion publishing"},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"publisher": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["ghi"]}, "publisher": {"id": [3]}},
                 "isbn": "ghi",
                 "publisher": {"id": 3, "name": "Rabbit publishers"},
                 "title": "The Rabbit Club",
             },
         ]
-        assert_resync_empty(sync, document.get("node", {}))
+        assert_resync_empty(sync, doc.get("node", {}))
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_update_non_primary_key_concurrent(
         self, data, book_cls, publisher_cls
     ):
         """Test sync update and then sync in concurrent mode."""
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": {
@@ -1012,7 +1038,7 @@ class TestParentSingleChildFkOnParent(object):
                 ],
             },
         }
-        sync = Sync(document)
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.search_client.refresh("testdb")
 
@@ -1020,19 +1046,19 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs == [
             {
-                "_meta": {"publisher": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "publisher": {"id": [1]}},
                 "isbn": "abc",
                 "publisher": {"id": 1, "name": "Tiger publishing"},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"publisher": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "publisher": {"id": [2]}},
                 "isbn": "def",
                 "publisher": {"id": 2, "name": "Lion publishing"},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"publisher": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["ghi"]}, "publisher": {"id": [3]}},
                 "isbn": "ghi",
                 "publisher": {"id": 3, "name": "Hop Bunny publishing"},
                 "title": "The Rabbit Club",
@@ -1069,37 +1095,40 @@ class TestParentSingleChildFkOnParent(object):
                             "pgsync.sync.Sync.status",
                             side_effect=noop,
                         ):
-                            sync.receive(NTHREADS_POLLDB)
+                            sync.receive()
                             sync.search_client.refresh("testdb")
 
         docs = search(sync.search_client, "testdb")
 
         assert docs == [
             {
-                "_meta": {"publisher": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "publisher": {"id": [1]}},
                 "isbn": "abc",
                 "publisher": {"id": 1, "name": "Tiger publishing"},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"publisher": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "publisher": {"id": [2]}},
                 "isbn": "def",
                 "publisher": {"id": 2, "name": "Lion publishing"},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"publisher": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["ghi"]}, "publisher": {"id": [3]}},
                 "isbn": "ghi",
                 "publisher": {"id": 3, "name": "Rabbit publishers"},
                 "title": "The Rabbit Club",
             },
         ]
-        assert_resync_empty(sync, document.get("node", {}))
+        assert_resync_empty(sync, doc.get("node", {}))
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
 
     def test_delete_concurrent(self, data, book_cls, publisher_cls):
         """Test sync delete and then sync in concurrent mode."""
-        document = {
+        doc = {
             "index": "testdb",
             "database": "testdb",
             "nodes": {
@@ -1118,7 +1147,7 @@ class TestParentSingleChildFkOnParent(object):
             },
         }
 
-        sync = Sync(document)
+        sync = Sync(doc)
         sync.search_client.bulk(sync.index, sync.sync())
         sync.search_client.refresh("testdb")
 
@@ -1126,19 +1155,19 @@ class TestParentSingleChildFkOnParent(object):
 
         assert docs == [
             {
-                "_meta": {"publisher": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "publisher": {"id": [1]}},
                 "isbn": "abc",
                 "publisher": {"id": 1, "name": "Tiger publishing"},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"publisher": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "publisher": {"id": [2]}},
                 "isbn": "def",
                 "publisher": {"id": 2, "name": "Lion publishing"},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"publisher": {"id": [3]}},
+                "_meta": {"book": {"isbn": ["ghi"]}, "publisher": {"id": [3]}},
                 "isbn": "ghi",
                 "publisher": {"id": 3, "name": "Hop Bunny publishing"},
                 "title": "The Rabbit Club",
@@ -1188,30 +1217,36 @@ class TestParentSingleChildFkOnParent(object):
                             "pgsync.sync.Sync.status",
                             side_effect=noop,
                         ):
-                            sync.receive(NTHREADS_POLLDB)
+                            sync.receive()
                             sync.search_client.refresh("testdb")
 
         docs = search(sync.search_client, "testdb")
 
         assert docs == [
             {
-                "_meta": {"publisher": {"id": [1]}},
+                "_meta": {"book": {"isbn": ["abc"]}, "publisher": {"id": [1]}},
                 "isbn": "abc",
                 "publisher": {"id": 1, "name": "Tiger publishing"},
                 "title": "The Tiger Club",
             },
             {
-                "_meta": {"publisher": {"id": [2]}},
+                "_meta": {"book": {"isbn": ["def"]}, "publisher": {"id": [2]}},
                 "isbn": "def",
                 "publisher": {"id": 2, "name": "Lion publishing"},
                 "title": "The Lion Club",
             },
             {
-                "_meta": {"publisher": {"id": [99]}},
+                "_meta": {
+                    "book": {"isbn": ["ghi"]},
+                    "publisher": {"id": [99]},
+                },
                 "isbn": "ghi",
                 "publisher": {"id": 99, "name": "Rabbit publishers"},
                 "title": "The Rabbit Club",
             },
         ]
-        assert_resync_empty(sync, document.get("node", {}))
+        assert_resync_empty(sync, doc.get("node", {}))
         sync.search_client.close()
+        sync.session.close()
+        sync.engine.dispose(close=True)
+        Singleton._instances = {}
