@@ -1,10 +1,17 @@
 """Trigger tests."""
+
 import pytest
+import sqlalchemy as sa
 
 from pgsync.base import Base
+from pgsync.settings import IS_MYSQL_COMPAT
 from pgsync.trigger import CREATE_TRIGGER_TEMPLATE
 
 
+@pytest.mark.skipif(
+    IS_MYSQL_COMPAT,
+    reason="Skipped because IS_MYSQL_COMPAT env var is set",
+)
 @pytest.mark.usefixtures("table_creator")
 class TestTrigger(object):
     """Trigger tests."""
@@ -21,6 +28,8 @@ DECLARE
   _indices TEXT [];
   _primary_keys TEXT [];
   _foreign_keys TEXT [];
+  _columns TEXT [];
+  _changed BOOLEAN;
 
 BEGIN
     -- database is also the channel name.
@@ -28,8 +37,7 @@ BEGIN
 
     IF TG_OP = 'DELETE' THEN
 
-        SELECT primary_keys, indices
-        INTO _primary_keys, _indices
+        SELECT primary_keys INTO _primary_keys
         FROM _view
         WHERE table_name = TG_TABLE_NAME;
 
@@ -43,11 +51,30 @@ BEGIN
     ELSE
         IF TG_OP <> 'TRUNCATE' THEN
 
-            SELECT primary_keys, foreign_keys, indices
-            INTO _primary_keys, _foreign_keys, _indices
+            SELECT primary_keys, foreign_keys, indices, columns
+            INTO _primary_keys, _foreign_keys, _indices, _columns
             FROM _view
             WHERE table_name = TG_TABLE_NAME;
 
+            -- normalize null to empty array
+            _columns := COALESCE(_columns, ARRAY[]::TEXT[]);
+
+            -- Only react if any _columns actually changed
+            IF TG_OP = 'UPDATE' THEN
+                SELECT EXISTS (
+                    SELECT 1
+                    FROM JSONB_EACH(TO_JSONB(NEW.*)) n
+                    JOIN JSONB_EACH(TO_JSONB(OLD.*)) o USING (key)
+                    WHERE n.key = ANY(_columns)
+                    AND n.value IS DISTINCT FROM o.value
+                )
+                INTO _changed;
+
+                IF NOT _changed THEN
+                    RETURN NEW;  -- skip notification; nothing relevant changed
+                END IF;
+            END IF;
+
             new_row = ROW_TO_JSON(NEW);
             new_row := (
                 SELECT JSONB_OBJECT_AGG(key, value)
@@ -107,7 +134,7 @@ $$ LANGUAGE plpgsql;
                 f"JOIN pg_attribute ON attrelid = indrelid AND attnum = ANY(indkey) "  # noqa E501
                 f"WHERE indrelid = '{table_name}'::regclass AND indisprimary"
             )
-            rows = pg_base.fetchall(query)[0]
+            rows = pg_base.fetchall(sa.text(query))[0]
             assert list(rows)[0] == primary_keys
 
     def test_trigger_foreign_key_function(self, connection):
@@ -129,7 +156,7 @@ $$ LANGUAGE plpgsql;
                 f"WHERE constraint_catalog=current_catalog AND "
                 f"table_name='{table_name}' AND position_in_unique_constraint NOTNULL "  # noqa E501
             )
-            rows = pg_base.fetchall(query)[0]
+            rows = pg_base.fetchall(sa.text(query))[0]
             if rows[0]:
                 assert sorted(rows[0]) == sorted(foreign_keys)
             else:
