@@ -1,13 +1,18 @@
 """Tests for `pgsync` package."""
 
-import psycopg2
 import pytest
 
 from pgsync.base import subtransactions
+from pgsync.node import Tree
+from pgsync.settings import IS_MYSQL_COMPAT
 
 from .testing_utils import assert_resync_empty, sort_list
 
 
+@pytest.mark.skipif(
+    IS_MYSQL_COMPAT,
+    reason="Skipped because IS_MYSQL_COMPAT env var is set",
+)
 @pytest.mark.usefixtures("table_creator")
 class TestUniqueBehaviour(object):
     """Unique behaviour tests."""
@@ -45,9 +50,6 @@ class TestUniqueBehaviour(object):
         ]
         with subtransactions(session):
             conn = session.connection().engine.connect().connection
-            conn.set_isolation_level(
-                psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT
-            )
             cursor = conn.cursor()
             channel = sync.database
             cursor.execute(f"UNLISTEN {channel}")
@@ -72,9 +74,6 @@ class TestUniqueBehaviour(object):
 
         with subtransactions(session):
             conn = session.connection().engine.connect().connection
-            conn.set_isolation_level(
-                psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT
-            )
             cursor = conn.cursor()
             channel = session.connection().engine.url.database
             cursor.execute(f"UNLISTEN {channel}")
@@ -101,8 +100,7 @@ class TestUniqueBehaviour(object):
             raise
 
         sync.redis.delete()
-        session.connection().engine.connect().close()
-        session.connection().engine.dispose()
+        session.connection().engine.dispose(close=True)
         sync.search_client.close()
 
     @pytest.fixture(scope="function")
@@ -217,9 +215,7 @@ class TestUniqueBehaviour(object):
         Test regular sync produces the correct result
         """
         sync.tree.__nodes = {}
-        sync.tree.__post_init__()
-        sync.nodes = nodes
-        sync.root = sync.tree.build(nodes)
+        sync.tree = Tree(sync.models, nodes, database=sync.database)
         docs = [sort_list(doc) for doc in sync.sync()]
         docs = sorted(docs, key=lambda k: k["_id"])
         assert docs == [
@@ -228,6 +224,7 @@ class TestUniqueBehaviour(object):
                 "_index": "testdb",
                 "_source": {
                     "_meta": {
+                        "book": {"isbn": ["abc"]},
                         "contact": {"id": [1, 2]},
                         "user": {"id": [1, 2]},
                     },
